#!/Users/jacky/tools/venv/bin/python3.9

# This app is developed for daily pdf manupilation 
from pikepdf import Pdf
from pdf2image import convert_from_path
from glob import glob
import argparse,os,img2pdf,shutil
from PIL import Image
from datetime import datetime

# convert img to pdf
def convertImg2Pdf(img_path):
    image = Image.open(img_path)
    pdf_bytes = img2pdf.convert(image.filename)
    pdf_path=img_path.split('.')[0]+".pdf"
    file = open(pdf_path, "wb")
    file.write(pdf_bytes)
    image.close()
    file.close()
    print(f"Successfully coverted {img_path} to pdf file {pdf_path}")

def batchCovertImg2Pdf(input):
    imgs=sorted(glob(input))
    for img in imgs:
        convertImg2Pdf(img)

def convert2img(pdfFile,folder="./",dpi=70):
    pages=convert_from_path(pdfFile,dpi)
    imgFileName=pdfFile.split(".")[0]
    i=0
    for page in pages:
        fn=folder+imgFileName+'{0:03}'.format(i)+".jpg"
        page.save(fn,'JPEG')
        i+=1
    print(f"Successfully converted pdf to {i} images...")

def pdf2pdf(pdfFile,outputFile=None,dpi=70):
    # make a temp folder
    now=datetime.now()
    tmpFolder=str(now.year)+str(now.month)+str(now.day)+str(now.hour)+str(now.minute)+str(now.second)
    os.mkdir(tmpFolder)
    # convert to imgs
    convert2img(pdfFile,tmpFolder+"/",dpi)
    # convert imgs to pdfs
    imgs=sorted(glob(tmpFolder+"/*.jpg"))
    for img in imgs:
        convertImg2Pdf(img)
    # delete all imgs
    remove="rm "+tmpFolder+"/*.jpg"
    os.system(remove)
    # merge pdfs to pdf
    targetName=outputFile or pdfFile.split(".")[0]+"_flatterned"+".pdf"
    merge(tmpFolder+"/*.pdf",targetName)
    # delete tmp folder
    shutil.rmtree(tmpFolder)
    print(f"{pdfFile} has been flatterned to {targetName}...")

def split(fileName):
    pdf=Pdf.open(fileName)
    outputName=fileName.split('.')[0]
    for n,page in enumerate(pdf.pages):
        dst=Pdf.new()
        dst.pages.append(page)
        dst.save(outputName+f'{n:03d}.pdf')
    pages=len(pdf.pages)
    print(f"Successfully splitted {fileName} to {pages} pdf.")

def merge(input,targetName):
    if type(input)==str:
        files=sorted(glob(input))
    elif type(input)==list:
        files=input
    else:
        files=None
        print('Input source error! it must be either filter string or a file list...')
        return
    pdf=Pdf.new()
    version=pdf.pdf_version
    for file in files:
        src=Pdf.open(file)
        version=max(version,src.pdf_version)
        pdf.pages.extend(src.pages)
    pdf.remove_unreferenced_resources()
    pdf.save(targetName)
    fileNumber=len(files)
    print(f'Successfully merged {fileNumber} files to {targetName}.')

def reverse(fileName):
    pdf=Pdf.open(fileName,allow_overwriting_input=True)
    pdf.pages.reverse()
    pdf.save()
    pages=len(pdf.pages)
    print(f"Successfully reversed {pages}.")

def append(source,target):
    pdf_source=Pdf.open(source)
    pdf_target=Pdf.open(target,allow_overwriting_input=True)
    pdf_target.pages.extend(pdf_source.pages)
    pdf_target.save()
    pages=len(pdf_source.pages)
    print(f'Successfully appended {pages} pages from {source} to {target}.')


# copy source pages from start page to end page to an existing file, or to append to a new file
def insert(source,target,startPage=0,endPage=-1,position=0):
    pdf_source=Pdf.open(source)
    if int(endPage)>len(pdf_source.pages):
        print(f"The end page {endPage} is bigger than the total pages {len(pdf_source.pages)}")
        return
    fileExist=os.path.isfile(target)
    if fileExist:
        pdf_target,position=Pdf.open(target,allow_overwriting_input=True),position 
    else:
        pdf_target,position=Pdf.new(),0

    if int(position)>len(pdf_target.pages):
        print(f"The position {position+1} is bigger than the total pages {len(pdf_target.pages)}")
        return
    pages=0
    initial_position=int(position)
    position=int(position)
    for page in range(int(startPage),int(endPage)):
        pdf_target.pages.insert(position,pdf_source.pages[page])
        position+=1
        pages+=1
    print(f'Total {pages} pages inserted to position from {initial_position} to {position}.')
    pdf_target.save() if fileExist else pdf_target.save(target)

def delete(fileName,start,end):
    pdf=Pdf.open(fileName,allow_overwriting_input=True)
    for page in range(int(end)-int(start)+1):
        del pdf.pages[end-page-1]
    pdf.save()
    print(f"Successfully deleted {end-start} pages from {start} to {end}\nNow {fileName} has {len(pdf.pages)} pages left.")



def main():
    parser=argparse.ArgumentParser(description="For munipulate pdfs: split, merge, reverse, rotate,img double way process...")
    
    parser.add_argument("-s", "--split", help="split pdf to sigle page with name: oringe+001.pdf")
    parser.add_argument("-f", "--filter", help="get pdf selection by using filter (exp: '*.pdf'. '' is must!), and merge to target file. (must followed by -t filename) ")
    parser.add_argument("-l", "--list", help="get pdf selection by list file names, and merge to target file. (must followed by -t filename) ",nargs="+")
    parser.add_argument("-r", "--reverse", help="reverse the pages of the pdf")
    parser.add_argument("-a", "--append", help="append the source pdf to target pdf, must followed by -t finename")
    parser.add_argument("-i", "--insert", help="insert pages with start page and end page from source to target file before the specified position. (must followed by -t filename) ",nargs="+")
    parser.add_argument("-d", "--delete", help="delete pages from start to end",nargs="+")
    parser.add_argument("-t", "--to", help="target pdf file name",nargs="+")
    parser.add_argument("-ip", "--img2pdf", help="img to pdf, source img file name")
    parser.add_argument("-bip", "--batchimg2pdf", help="get picture selection by using filter (exp: '*.jpg'. '' is must!) ")
    parser.add_argument("-pi", "--pdf2img", help="pdf to img source pdf file name. img files will use pdf's name with .jpg")
    parser.add_argument("-pp", "--pdf2pdf", help="source pdf file name. convert pdf to img and then to pdf. Make it flatterned")
    parser.add_argument("-dpi", "--dpi", help="set dpi value")

    args = parser.parse_args()

    if args.split:
        split(args.split)
        return 
    if args.filter and args.to:
        merge(args.filter,args.to[0])
        return
    if args.list and args.to:
        merge(args.list,args.to[0])
        return
    if args.reverse:
        reverse(args.reverse)
        return
    if args.append and args.to:
        append(args.append,args.to[0])
        return
    if args.insert and args.to:
        start=int(args.insert[1])-1
        end=int(args.insert[2])
        position=int(args.to[1])-1 if len(args.to)>1 else 0
        insert(args.insert[0],args.to[0],start,end,position)
        return
    if args.delete:
        start=int(args.delete[1])
        end=int(args.delete[2])
        delete(args.delete[0],start,end)
        return 

    if args.img2pdf:
        convertImg2Pdf(args.img2pdf)
        return 

    if args.batchimg2pdf:
        batchCovertImg2Pdf(args.batchimg2pdf)
        return

    if args.pdf2img:
        convert2img(args.pdf2img)
        return 
    
    if args.pdf2pdf:
        dpi=args.dpi or 70
        pdf2pdf(args.pdf2pdf,args.to[0]) if args.to else  pdf2pdf(args.pdf2pdf,dpi=dpi)
        return 

if __name__=="__main__":
    main()



import xlsxwriter


TITLE_FORMAT = {
    "font_size": 24,
    "bold": True,
    "font_color": "000000",
    "align": "center",
    "valign": "middle",
    "bg_color": "C0C0C0",
    "border": 1
}

COLUMN_TITLE_FORMAT = {
    "font_size": 14,
    "bold": True,
    "font_color": "000000",
    "align": "center",
    "valign": "middle",
    "bg_color": "C0C0C0",
    "border": 1
}

VARIABLE_TITLE_FORMAT = {
    "font_size": 14,
    "font_color": "000000",
    "align": "center",
    "valign": "middle",
    "bg_color": "C0C0C0",
    "border": 1
}

DESCRIPTION_FORMAT = {
    "font_size": 14,
    "font_color": "000000",
    "align": "right",
    "valign": "vcenter",
    "bg_color": "f4f3ee",
    "border": 1
}

VALUE_FORMAT = {
    "font_size": 14,
    "font_color": "719c75",
    "align": "left",
    "valign": "vcenter",
    "locked": False,
    "border": 1,
    "text_wrap": 1
}

COMMENT_FORMAT = {
    "font_size": 12,
    # "font_color": "CCFFCC",
    "author": "Jacky Zhang",
    "width": 200,
    "heigth": 400
}

COLUMN_WIDTH = {
    "noc_code": 10,
    "date": 12,
    'name': 20,
    'fullname': 30,
    'country': 20,
    'relation': 15,
    'address': 30,
    'type': 20,
    'email': 25,
    'title': 20,
    'city': 15,
    'province': 15,
    'status': 20,
    'field_of_study': 20,
    'duties': 50,
    'company_brief': 30,
    'brief': 30
}

special_format = {
    'table-eraddress': {
        'variable_type': {'locked': True, "bg_color": "f4f3ee"},
        'display_type': {'locked': True, "bg_color": "f4f3ee"},
        'country': {'locked': True, "bg_color": "f4f3ee"}
    },
    "table-contact": {
        "contact_variable": {'locked': True, "bg_color": "f4f3ee"},
        "contact_type": {'locked': True, "bg_color": "f4f3ee"}
    },
    'table-employment': {
        "noc_code": {"num_format": "@"}
    },
    'table-finance': {
        "total_asset": {"num_format": "$#,##0"},
        "net_asset": {"num_format": "$#,##0"},
        "revenue": {"num_format": "$#,##0"},
        "net_income": {"num_format": "$#,##0"},
        "retained_earning": {"num_format": "$#,##0"}
    },
    'table-employee_list': {
        'noc': {"num_format": "@"},
        "percentage_to_median": {"num_format": "0.0%"}
    },
    'info-joboffer': {
        'noc': {"num_format": "@"},
    },
    "table-personid": {
        "variable_type": {'locked': True, "bg_color": "f4f3ee"},
        "display_type": {'locked': True, "bg_color": "f4f3ee"}
    },
    "type-assumption": {
        "noc_code": {"num_format": "@"}
    },
    "table-phone": {
        "variable_type": {'locked': True, "bg_color": "f4f3ee"},
        "display_type": {'locked': True, "bg_color": "f4f3ee"}
    },
    "table-address": {
        "variable_type": {'locked': True, "bg_color": "f4f3ee"},
        "display_type": {'locked': True, "bg_color": "f4f3ee"}
    },
    'table-employment': {
        'noc_code': {"num_format": "@"},
        "share_percentage": {"num_format": "0.0%"}
    }

}



country_residence={
    "Select": "null",
    "Afghanistan": "252",
    "Albania": "081",
    "Algeria": "131",
    "Andorra": "082",
    "Angola": "151",
    "Anguilla": "620",
    "Antigua And Barbuda": "621",
    "Argentina": "703",
    "Armenia": "049",
    "Aruba": "658",
    "Australia": "305",
    "Austria": "011",
    "Azerbaijan": "050",
    "Azores": "035",
    "Bahamas": "622",
    "Bahrain": "253",
    "Bangladesh": "212",
    "Barbados": "610",
    "Belarus": "051",
    "Belgium": "012",
    "Belize": "541",
    "Benin": "160",
    "Bermuda": "601",
    "Bhutan": "254",
    "Bolivia": "751",
    "Bosnia and Herzegovina": "048",
    "Botswana": "153",
    "Brazil": "709",
    "British Virgin Islands": "633",
    "Brunei Darussalam": "255",
    "Bulgaria": "083",
    "Burkina Faso": "188",
    "Burma (Myanmar)": "241",
    "Burundi": "154",
    "Cabo Verde": "911",
    "Cambodia": "256",
    "Cameroon": "155",
    "Canada": "511",
    "Canary Islands": "039",
    "Cayman Islands": "624",
    "Central African Republic": "157",
    "Chad": "156",
    "Channel Islands": "009",
    "Chile": "721",
    "China": "202",
    "China (Hong Kong SAR)": "200",
    "China (Macao SAR)": "198",
    "Colombia": "722",
    "Comoros": "905",
    "Cook Islands": "840",
    "Costa Rica": "542",
    "Croatia": "043",
    "Cuba": "650",
    "Cyprus": "221",
    "Czech Republic": "015",
    "Democratic Rep. of Congo": "158",
    "Denmark": "017",
    "Djibouti": "183",
    "Dominica": "625",
    "Dominican Republic": "651",
    "East Timor": "916",
    "Ecuador": "753",
    "Egypt": "101",
    "El Salvador": "543",
    "England": "002",
    "Equatorial Guinea": "178",
    "Eritrea": "162",
    "Estonia": "018",
    "Ethiopia": "161",
    "Falkland Islands": "912",
    "Federated States of Micronesia": "835",
    "Fiji": "801",
    "Finland": "021",
    "Fr. South. and Antarctic Lands": "821",
    "France": "022",
    "French Guiana": "754",
    "French Polynesia": "845",
    "Gabon": "163",
    "Gambia": "164",
    "Georgia": "052",
    "Germany, Federal Republic Of": "024",
    "Ghana": "165",
    "Gibraltar": "084",
    "Greece": "025",
    "Greenland": "521",
    "Grenada": "626",
    "Guadeloupe": "653",
    "Guam": "832",
    "Guatemala": "544",
    "Guinea": "166",
    "Guinea-Bissau": "167",
    "Guyana": "711",
    "Haiti": "654",
    "Honduras": "545",
    "Hong Kong": "204",
    "Hungary": "026",
    "Iceland": "085",
    "India": "205",
    "Indonesia": "222",
    "Iran": "223",
    "Iraq": "224",
    "Ireland": "027",
    "Israel": "206",
    "Italy": "028",
    "Ivory Coast": "169",
    "Jamaica": "602",
    "Japan": "207",
    "Jordan": "225",
    "Kazakhstan": "053",
    "Kenya": "132",
    "Kiribati": "831",
    "Korea, North (DPRK)": "257",
    "Korea, South": "258",
    "Kosovo": "064",
    "Kuwait": "226",
    "Kyrgyzstan": "054",
    "Laos": "260",
    "Latvia": "019",
    "Lebanon": "208",
    "Lesotho": "152",
    "Liberia": "170",
    "Libya": "171",
    "Liechtenstein": "086",
    "Lithuania": "020",
    "Luxembourg": "013",
    "Macao": "261",
    "Macedonia": "070",
    "Madagascar": "172",
    "Madeira": "036",
    "Malawi": "111",
    "Malaysia": "242",
    "Maldives": "901",
    "Mali": "173",
    "Malta": "030",
    "Marshall Islands": "834",
    "Martinique": "655",
    "Mauritania": "174",
    "Mauritius": "902",
    "Mayotte": "906",
    "Mexico": "501",
    "Moldova": "055",
    "Monaco": "087",
    "Mongolia": "262",
    "Montenegro": "063",
    "Montserrat": "627",
    "Morocco": "133",
    "Mozambique": "175",
    "Namibia": "122",
    "Nauru": "341",
    "Nepal": "264",
    "Netherlands Antilles, The": "652",
    "Netherlands, The": "031",
    "Nevis": "628",
    "New Caledonia": "822",
    "New Zealand": "339",
    "Nicaragua": "546",
    "Niger": "176",
    "Nigeria": "177",
    "North Vietnam": "271",
    "Northern Ireland": "006",
    "Northern Mariana Islands": "830",
    "Norway": "032",
    "Oman": "263",
    "Pakistan": "209",
    "Palestinian Authority": "213",
    "Panama": "547",
    "Papua New Guinea": "342",
    "Paraguay": "755",
    "Peru": "723",
    "Philippines": "227",
    "Pitcairn Islands": "842",
    "Poland": "033",
    "Portugal": "034",
    "Puerto Rico": "656",
    "Qatar": "265",
    "Republic of Congo": "159",
    "Republic of Palau": "836",
    "Réunion": "903",
    "Romania": "088",
    "Russia": "056",
    "Rwanda": "179",
    "Saint Helena": "915",
    "Saint Kitts and Nevis": "629",
    "Saint Lucia": "630",
    "Saint Pierre and Miquelon": "531",
    "Saint-Martin": "415",
    "Samoa": "844",
    "Samoa, American": "843",
    "San Marino": "089",
    "Sao Tome and Principe": "914",
    "Saudi Arabia": "231",
    "Scotland": "007",
    "Senegal": "180",
    "Serbia, Republic Of": "062",
    "Seychelles": "904",
    "Sierra Leone": "181",
    "Singapore": "246",
    "Slovakia": "016",
    "Slovenia": "047",
    "Solomon Islands": "824",
    "Somalia": "182",
    "South Africa, Republic Of": "121",
    "South Sudan": "189",
    "Spain": "037",
    "Sri Lanka": "201",
    "St. Vincent and the Grenadines": "631",
    "Sudan": "185",
    "Suriname": "752",
    "Swaziland": "186",
    "Sweden": "040",
    "Switzerland": "041",
    "Syria": "210",
    "Taiwan": "203",
    "Tajikistan": "057",
    "Tanzania": "130",
    "Thailand": "267",
    "Togo": "187",
    "Tonga": "846",
    "Trinidad and Tobago": "605",
    "Tunisia": "135",
    "Turkey": "045",
    "Turkmenistan": "058",
    "Turks and Caicos Islands": "632",
    "Tuvalu": "826",
    "Uganda": "136",
    "Ukraine": "059",
    "United Arab Emirates": "280",
    "United States of America": "461",
    "Unknown": "000",
    "Uruguay": "724",
    "Uzbekistan": "060",
    "Vanuatu": "823",
    "Vatican City State": "090",
    "Venezuela": "725",
    "Vietnam": "270",
    "Virgin Islands, U.S.": "657",
    "Wales": "008",
    "Wallis and Futuna Is., Terr.": "841",
    "Western Sahara": "184",
    "Yemen": "273",
    "Zambia": "112",
    "Zimbabwe": "113"
}





# Data infrastructure for NOC wage and outlook 
area_info = [
    ["Canada", "Canada", 0],
    ["Newfoundland and Labrador", "Newfoundland and Labrador", 1],
    ["Newfoundland and Labrador", "Avalon Peninsula Region", 2],
    ["Newfoundland and Labrador", "Notre Dame-Central-Bonavista Bay Region", 3],
    ["Newfoundland and Labrador", "South Coast–Burin Peninsula Region", 4],
    ["Newfoundland and Labrador", "West Coast–Northern Peninsula–Labrador Region", 5],
    ["Prince Edward Island", "Prince Edward Island", 6],
    ["Nova Scotia", "Nova Scotia", 7],
    ["Nova Scotia", "Annapolis Valley Region", 8],
    ["Nova Scotia", "Cape Breton Region", 9],
    ["Nova Scotia", "Halifax Region", 10],
    ["Nova Scotia", "North Shore Region", 11],
    ["Nova Scotia", "Southern Region", 12],
    ["New Brunswick", "New Brunswick", 13],
    ["New Brunswick", "Campbellton–Miramichi Region", 14],
    ["New Brunswick", "Edmundston–Woodstock Region", 15],
    ["New Brunswick", "Fredericton–Oromocto", 16],
    ["New Brunswick", "Moncton–Richibucto Region", 17],
    ["New Brunswick", "Saint John–St. Stephen Region", 18],
    ["Quebec", "Quebec", 19],
    ["Quebec", "Abitibi-Témiscamingue Region", 20],
    ["Quebec", "Bas-Saint-Laurent Region", 21],
    ["Quebec", "Capitale-Nationale Region", 22],
    ["Quebec", "Centre-du-Québec Region", 23],
    ["Quebec", "Chaudière-Appalaches Region", 24],
    ["Quebec", "Côte-Nord Region", 25],
    ["Quebec", "Estrie Region", 26],
    ["Quebec", "Gaspésie–Îles-de-la-Madeleine Region", 27],
    ["Quebec", "Lanaudière Region", 28],
    ["Quebec", "Laurentides Region", 29],
    ["Quebec", "Laval Region", 30],
    ["Quebec", "Mauricie Region", 31],
    ["Quebec", "Montréal Region", 32],
    ["Quebec", "Montérégie Region", 33],
    ["Quebec", "Nord-du-Québec Region", 34],
    ["Quebec", "Outaouais Region", 35],
    ["Quebec", "Saguenay–Lac-Saint-Jean Region", 36],
    ["Ontario", "Ontario", 37],
    ["Ontario", "Hamilton–Niagara Peninsula Region", 38],
    ["Ontario", "Kingston–Pembroke Region", 39],
    ["Ontario", "Kitchener–Waterloo–Barrie Region", 40],
    ["Ontario", "London Region", 41],
    ["Ontario", "Muskoka–Kawarthas Region", 42],
    ["Ontario", "Northeast Region", 43],
    ["Ontario", "Northwest Region", 44],
    ["Ontario", "Ottawa Region", 45],
    ["Ontario", "Stratford–Bruce Peninsula Region", 46],
    ["Ontario", "Toronto Region", 47],
    ["Ontario", "Windsor-Sarnia Region", 48],
    ["Manitoba", "Manitoba", 49],
    ["Manitoba", "Interlake Region", 50],
    ["Manitoba", "North Central Region", 51],
    ["Manitoba", "North Region", 52],
    ["Manitoba", "Parklands Region", 53],
    ["Manitoba", "South Central Region", 54],
    ["Manitoba", "Southeast Region", 55],
    ["Manitoba", "Southwest Region", 56],
    ["Manitoba", "Winnipeg Region", 57],
    ["Saskatchewan", "Saskatchewan", 58],
    ["Saskatchewan", "Northern Region", 59],
    ["Saskatchewan", "Prince Albert Region", 60],
    ["Saskatchewan", "Regina–Moose Mountain Region", 61],
    ["Saskatchewan", "Saskatoon–Biggar Region", 62],
    ["Saskatchewan", "Swift Current–Moose Jaw Region", 63],
    ["Saskatchewan", "Yorkton–Melville Region", 64],
    ["Alberta", "Alberta", 65],
    ["Alberta", "Athabasca–Grande Prairie–Peace River Region", 66],
    ["Alberta", "Banff–Jasper–Rocky Mountain House Region", 67],
    ["Alberta", "Calgary Region", 68],
    ["Alberta", "Camrose–Drumheller Region", 69],
    ["Alberta", "Edmonton Region", 70],
    ["Alberta", "Lethbridge–Medicine Hat Region", 71],
    ["Alberta", "Red Deer Region", 72],
    ["Alberta", "Wood Buffalo–Cold Lake Region", 73],
    ["British Columbia", "British Columbia", 74],
    ["British Columbia", "Cariboo Region", 75],
    ["British Columbia", "Kootenay Region", 76],
    ["British Columbia", "Lower Mainland–Southwest Region", 77],
    ["British Columbia", "Nechako Region", 78],
    ["British Columbia", "North Coast Region", 79],
    ["British Columbia", "Northeast Region", 80],
    ["British Columbia", "Thompson–Okanagan Region", 81],
    ["British Columbia", "Vancouver Island and Coast Region", 82],
    ["Yukon Territory", "Yukon Territory", 83],
    ["Northwest Territories", "Northwest Territories", 84],
    ["Nunavut", "Nunavut", 85]]

def getIndexByAreaName(area_name):
    index=0
    for area in area_info:
        if area_name==area[1]:
            return index
        index+=1

#col 1 is index for prince in wage/outlook search, col 2,3 is area range, 4 is index of province for provincial median wage search
prov_area=[     
    [1,2,5,4],    # Newfoundland and Labrador
    [6,6,6,9],    # Prince Edward Island
    [7,8,12,6],   # Nova Scotia
    [13,14,18,3], # New Brunswick
    [19,20,36,10], # Quebec
    [37,38,48,8], # Ontario
    [49,50,57,2], # Manitoba
    [58,59,64,11], # Saskatchewan
    [65,66,73,0], # Alberta
    [74,75,82,1], # British Columbia
    [83,83,83,12], # Yukon Territory
    [84,84,84,5], # Northwest Territories
    [85,85,85,7]  # Nunavut
]

# for wage/outlook
def get_prov_index(area_code):  
    for pa in prov_area:
        if (area_code>=pa[1]) & (area_code<=pa[2]):
            return pa[0] 

# for provincial median wage https://www.canada.ca/en/employment-social-development/services/foreign-workers/service-tables.html
def get_prov_index2(area_code):  
    for pa in prov_area:
        if (int(area_code)>=pa[1]) & (int(area_code)<=pa[2]):
            return pa[3] 

def get_prov_name(area_code):
    for ai in area_info:
        if ai[2]==int(area_code):
            return ai[0]

def get_area_name(area_code):
    for ai in area_info:
        if ai[2]==int(area_code):
            return ai[1]


from engine.scoringbc import ScoringBCPNP

pa1={
    "noc":'1111',
    "is_working_in_the_position":True,
    "has_one_year_canadian_experience":True,
    "hourly_rate":27,
    "working_hours":40,
    "work_experience":48,
    "region":'Greater Vancouver',
    "education":'Non-trades certification or Diploma',
    "education_bonus":'ECA',
    "clb":6
}

b=ScoringBCPNP(**pa1)
print(pa1)
print(b.total_point)
print(b.noc_level_point,'noc level point')
print(b.noc_00_point,'noc_00 point')
print(b.wage_point,'wage_points')
print(b.work_experience_point,'work_experience_point')
print(b.working_in_the_position_point,'working_in_the_position_point')
print(b.one_year_Canadian_work_experience_point,'one_year_Canadian_work_experience')
print(b.educaiton_level_point,'educaiton_level_points')
print(b.education_bonus_point,'education_bonus_points')
print(b.region_point,'region_points')
print(b.language_level_point,'language_level_points')


# This app is ONLY used for formatting source.excel's data
import xlsxwriter
from copy import deepcopy
from source.comment import comment
from source.formatdefinition import special_format, COLUMN_WIDTH, VALUE_FORMAT, TITLE_FORMAT, DESCRIPTION_FORMAT, COLUMN_TITLE_FORMAT, VARIABLE_TITLE_FORMAT, COMMENT_FORMAT
from importlib import import_module

# from source.validation import validation
validation=import_module("source.validation").validation

class ExcelWritter():
    def __init__(self, input_excel_obj, out_excel):
        self.workbook = xlsxwriter.Workbook(
            out_excel, {'default_date_format': "yyyy-mm-dd"})
        self.workbook.set_size(1440, 1640)
        self.excel = deepcopy(input_excel_obj)
        self.protection_options = {
            'objects':               False,
            'scenarios':             False,
            'format_cells':          False,
            'format_columns':        True,
            'format_rows':           True,
            'insert_columns':        False,
            'insert_rows':           False,
            'insert_hyperlinks':     False,
            'delete_columns':        False,
            'delete_rows':           False,
            'select_locked_cells':   False,
            'sort':                  False,
            'autofilter':            False,
            'pivot_tables':          False,
            'select_unlocked_cells': True,
        }

    def _createTitle(self, worksheet, title, column_number):
        format = self.workbook.add_format(TITLE_FORMAT)
        worksheet.merge_range(0, 0, 0, column_number-1, None, format)
        worksheet.write(0, 0, title, format)

    def _createColumnTitles(self, worksheet, titles):
        for index, title in enumerate(titles):
            format = self.workbook.add_format(COLUMN_TITLE_FORMAT)
            worksheet.write(1, index, title, format)

    def _createColumnVariables(self, worksheet, variables):
        for index, variable in enumerate(variables):
            format = self.workbook.add_format(COLUMN_TITLE_FORMAT)
            worksheet.write(2, index, variable, format)
        worksheet.set_row(2, 0, self.workbook.add_format({"hidden": True}))

    def _createSheetContents(self, worksheet, sheetcontent, sheet_name):
        # 获得所有variable, tag, description, and value
        row = 3  # initial row is 3
        for row_obj in sheetcontent.data.values():
            worksheet.write(row, 0, row_obj.variable)
            worksheet.write(row, 1, row_obj.tag)
            worksheet.write(row, 2, row_obj.description,
                            self.workbook.add_format(DESCRIPTION_FORMAT))
            # write value with special format and default foramt
            if sheet_name in special_format.keys() and row_obj.variable in special_format[sheet_name] and special_format[sheet_name][row_obj.variable]:
                new_format = self.workbook.add_format(
                    {**VALUE_FORMAT, **special_format[sheet_name][row_obj.variable]})
            elif "_date" in row_obj.variable:
                new_format = self.workbook.add_format({
                    **VALUE_FORMAT, **{'num_format': 'yyyy-mm-dd'}})
            else:
                new_format = self.workbook.add_format(VALUE_FORMAT)
            worksheet.write(row, 3, row_obj.value, new_format)

            # add comment
            if sheet_name in comment.keys() and row_obj.variable in comment[sheet_name] and comment[sheet_name][row_obj.variable]:
                worksheet.write_comment(
                    row, 2, comment[sheet_name][row_obj.variable], COMMENT_FORMAT)

            # add data validation
            if sheet_name in validation.keys() and row_obj.variable in validation[sheet_name] and validation[sheet_name][row_obj.variable]:
                worksheet.data_validation(
                    row, 3, row, 3, validation[sheet_name][row_obj.variable])

            row += 1

    def _createTableContents(self, worksheet, tablecontent):
        table_name = "table-"+tablecontent.name
        # 获得所有table data and write
        r = 3
        for row in tablecontent.data:
            c = 0

            # Iterate over the data and write it out row by row.
            for variable, value in row.__dict__.items():
                if "_date" in variable:
                    new_format = self.workbook.add_format(
                        {**VALUE_FORMAT, **{'num_format': 'yyyy-mm-dd'}})
                elif table_name in special_format.keys() and variable in special_format[table_name] and special_format[table_name][variable]:
                    new_format = self.workbook.add_format(
                        {**VALUE_FORMAT, **special_format[table_name][variable]})
                else:
                    new_format = self.workbook.add_format(VALUE_FORMAT)
                worksheet.write(
                    r, c, value, new_format)
                c += 1
            c = 0
            r += 1

    def createSheet(self, protection, password):
        # 获取所有sheet表，column titles, column variables
        for sheet, sheetcontent in self.excel.sheets.items():
            sheet_name = 'info-'+sheet
            worksheet = self.workbook.add_worksheet(sheet_name)
            # create sheet
            self._createTitle(worksheet, self.excel.sheets[sheet].sheet_title, len(
                self.excel.sheets[sheet].column_variables))
            self._createColumnTitles(
                worksheet, self.excel.sheets[sheet].column_titles)
            self._createColumnVariables(
                worksheet, self.excel.sheets[sheet].column_variables)
            self._createSheetContents(worksheet, sheetcontent, sheet_name)
            # format sheet in sheet level
            worksheet.set_column(
                'A:B', 15, None, {'hidden': True})
            worksheet.set_column(
                'C:C', 70, None, {'text_wrap': True})
            worksheet.set_column(
                'D:D', 70, None, {'text_wrap': True})
            # protect the sheet
            if protection:
                worksheet.protect(password, self.protection_options)

    # 获取所有table表 column titles, column variables
    def createTable(self, protection, password):
        for table, tablecontent in self.excel.tables.items():
            table_name = 'table-'+table
            worksheet = self.workbook.add_worksheet(table_name)
            variables = tablecontent.column_variables
            titles = [tablecontent.column_titles[x]
                      for x in variables]
            # create table
            self._createTitle(
                worksheet, self.excel.tables[table].sheet_title, len(variables))
            self._createColumnTitles(worksheet, titles)
            self._createColumnVariables(worksheet, variables)
            self._createTableContents(worksheet, tablecontent)
            # format table in sheet level
            for index, variable in enumerate(variables):

                # 2. set date format
                if "_date" in variable:
                    the_format = self.workbook.add_format(
                        {'num_format': 'yyyy-mm-dd', 'align': "center"})
                    worksheet.set_column(index, index, 12, the_format)
                # 3. add validation
                if table_name in validation.keys() and variable in validation[table_name] and validation[table_name][variable]:
                    worksheet.data_validation(
                        3, index, 303, index, validation[table_name][variable])
                # 4. add comment
                if table_name in comment.keys() and variable in comment[table_name] and comment[table_name][variable]:
                    worksheet.write_comment(
                        1, index, comment[table_name][variable], COMMENT_FORMAT)
                # 5. set column with special format and default foramt
                if table_name in special_format.keys() and variable in special_format[table_name] and special_format[table_name][variable]:
                    new_format = self.workbook.add_format(
                        {**VALUE_FORMAT, **special_format[table_name][variable], "border": 0})
                else:
                    new_format = self.workbook.add_format(
                        {**VALUE_FORMAT, "border": 0})
                # set column width
                if variable in COLUMN_WIDTH.keys():
                    worksheet.set_column(
                        index, index, COLUMN_WIDTH[variable], new_format)
                else:
                    worksheet.set_column(index, index, 15, new_format)
            # protect the sheet
            if protection:
                worksheet.protect(password, self.protection_options)

    def create(self, protection=True, password=""):
        self.createSheet(protection, password)
        self.createTable(protection, password)
        self.workbook.close()




from webform.models.definition import Action
from webform.models.bcpnpmodel_reg import BcpnpModelReg, BcpnpEEModelReg
from model.common.id import IDs
from model.common.phone import Phones
from model.common.address import Addresses
from webform.bcpnp.data import country_map
from typing import Union


class Register:
    def __init__(self, person: Union[BcpnpModelReg, BcpnpEEModelReg]):
        self.person = person

    def register(self):
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Create your profile",
                "actions": self.actions,
                "id": "#form > div.row > div > input",
            },
            {
                "action_type": Action.WebPage.value,
                "page_name": "Confirm your profile",
                "actions": [
                    {
                        "action_type": Action.Checkbox.value,
                        "label": "Confirm your information",
                        "id": "#fieldset-actions > div > div > label > input[type=checkbox]:nth-child(2)",
                    }
                ],
                "id": "#fieldset-actions > div > input.btn.btn-primary",
            },
        ]

    @property
    def gotoRegister(self):
        return [
            {
                "action_type": Action.GotoPage.value,
                "url": "https://www.pnpapplication.gov.bc.ca/user/register",
            }
        ]

    @property
    def login(self):
        return [
            {
                "action_type": Action.Input.value,
                "label": "Email address",
                "id": "#email",
                "value": self.person.personal.email,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Confirm Email address",
                "id": "#emailConfirmation",
                "value": self.person.personal.email,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "User ID",
                "id": "#userId",
                "value": self.person.personal.user_id,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Password",
                "id": "#pass",
                "value": self.person.personal.password,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Password confirmation",
                "id": "#passConfirmation",
                "value": self.person.personal.password,
                "required": True,
                "length": 100,
            },
        ]

    @property
    def security(self):
        actions = []
        question_answer_pairs = {
            "Your country": "China",
            "Your province": "Jiangsu",
            "Where is your city": "Nanjing",
        }
        i = 0
        for key, value in question_answer_pairs.items():
            # select q
            actions += [
                {
                    "action_type": Action.Select.value,
                    "label": f"Security questions {i}",
                    "value": "_other",
                    "id": f"#question{i}",
                }
            ]
            # own question
            actions += [
                {
                    "action_type": Action.Input.value,
                    "label": f"Enter your own question {i}",
                    "value": key,
                    "id": f"#questionOther{i}",
                    "required": True,
                    "length": 100,
                }
            ]
            # answer
            actions += [
                {
                    "action_type": Action.Input.value,
                    "label": f"Enter your answer {i}",
                    "value": value,
                    "id": f"#answer{i}",
                    "required": True,
                    "length": 100,
                }
            ]
            # wait for 2 seconds
            actions += [{"action_type": Action.Wait.value, "duration": 2000}]
            i += 1
        return actions

    @property
    def personal_info(self):
        name = [
            {
                "action_type": Action.Input.value,
                "label": "Family name",
                "value": self.person.personal.last_name,
                "id": "#lastName",
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Given name",
                "value": self.person.personal.first_name,
                "id": "#firstName",
                "required": True,
                "length": 100,
            },
        ]

        has_used_name = [
            {
                "action_type": Action.Radio.value,
                "label": "Have you ever used any other name?",
                "id": "#form > fieldset:nth-child(4) > div > div:nth-child(3) > div > div:nth-child(3) > label > input[type=radio]"
                if self.person.personal.used_first_name
                else "#otherNames",
            }
        ]

        used_name = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Used family name",
                    "value": self.person.personal.used_last_name,
                    "id": "#otherLastNames",
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Used given name",
                    "value": self.person.personal.used_first_name,
                    "id": "#otherFirstNames",
                    "required": True,
                    "length": 100,
                },
            ]
            if self.person.personal.used_first_name
            else []
        )

        dob = [
            {
                "action_type": Action.Input.value,
                "label": "Date of birth",
                "value": self.person.personal.dob.strftime("%Y-%m-%d"),
                "id": "#birthDate",
                "required": True,
                "length": 10,
            }
        ]

        birth_place = [
            {
                "action_type": Action.Select.value,
                "label": "Country of birth",
                "id": "#birthCountry",
                "value": country_map[self.person.personal.country_of_birth],
            },
            {
                "action_type": Action.Input.value,
                "label": "City of birth",
                "id": "#birthCity",
                "value": self.person.personal.place_of_birth,
            },
        ]

        gender = [
            {
                "action_type": Action.Select.value,
                "label": "Gender",
                "id": "#sex",
                "value": self.person.personal.sex,
            }
        ]

        return name + has_used_name + used_name + dob + birth_place + gender

    @property
    def passport(self):
        passport = IDs(self.person.personid).passport
        return [
            {
                "action_type": Action.Input.value,
                "label": "Passport number",
                "value": passport.number,
                "id": "#passportNo",
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Select.value,
                "label": "Passport country",
                "id": "#passportCountry",
                "value": passport.country,
            },
            {
                "action_type": Action.Input.value,
                "label": "Issue date",
                "value": passport.issue_date.strftime("%Y-%m-%d"),
                "id": "#passportIssueDate",
                "required": True,
                "length": 10,
            },
            {
                "action_type": Action.Input.value,
                "label": "Date of passport expiry",
                "value": passport.expiry_date.strftime("%Y-%m-%d"),
                "id": "#passportExpiryDate",
                "required": True,
                "length": 10,
            },
        ]

    @property
    def contact(self):
        phone = Phones(self.person.phone).PreferredPhone
        return [
            {
                "action_type": Action.Input.value,
                "label": "Phone number",
                "value": str(phone.number),
                "id": "#phone",
                "required": True,
                "length": 100,
            }
        ]

    @property
    def address(self):
        address = Addresses(self.person.address).residential
        return [
            {
                "action_type": Action.Select.value,
                "label": "Country",
                "id": "#country",
                "value": country_map[address.country],
            },
            {
                "action_type": Action.Input.value,
                "label": "Address line",
                "value": address.line1,
                "id": "#addressLine",
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "City",
                "value": address.city,
                "id": "#city",
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Province",
                "value": address.province,
                "id": "#state",
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Postal code",
                "value": address.post_code,
                "id": "#postalCode",
                "required": True,
                "length": 100,
            },
        ]

    @property
    def additional(self):
        return [
            {
                "action_type": Action.Select.value,
                "label": "How did you learn about BCPNP",
                "id": "#bcpnpKnowledgeSource",
                "value": "CIC",
            }
        ]

    @property
    def continue_confirm(self):
        return [
            {
                "action_type": Action.Continue.value,
                "message": "Do you want to continue?",
            }
        ]

    @property
    def actions(self):
        return (
            self.gotoRegister
            + self.login
            + self.security
            + self.personal_info
            + self.passport
            + self.contact
            + self.address
            + self.additional
        )



from .context import DATADIR
from model.common.commonmodel import CommonModel
from model.bcpnp.data import General
from model.common.jobposition import Position
from model.common.wordmaker import WordMaker
import os

        
class CompanyInfoModel(CommonModel):
    general:General
    position:Position
    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self,excels=None,output_excel_file=None):
        if output_excel_file:
            excels=self.getExcels(['excel/er.xlsx'])
        else:
            if excels is None and len(excels)==0:
                raise ValueError('You must input excel file list as source data for validation')
        # call parent class for validating
        super().__init__(excels,output_excel_file,globals())

class CompanyInfoDocxAdaptor():
    def __init__(self,employer_training_obj:CompanyInfoModel):
        self.employer_training_obj=employer_training_obj
        
    def make(self, output_docx):
        template_path=os.path.abspath(os.path.join(DATADIR,"word/bcpnp_company_information.docx")) 
        wm = WordMaker(template_path, self.employer_training_obj, output_docx)
        wm.make()
        


from typing import List
from model.common.trperson import Family
from model.common.commonmodel import CommonModel

"""
Program model for temporary resident visa. Get and validate info for forms: imm5257, imm0104, imm5257b_1, and imm5645
"""
class M5645Model(CommonModel):
    family:List[Family]
    
    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self,excels=None,output_excel_file=None):
        if output_excel_file:
           excels=self.getExcels(["excel/pa.xlsx"])
        else:
            if excels is None and len(excels)==0:
                raise ValueError('You must input excel file list as source data for validation')
        super().__init__(excels,output_excel_file,globals())




from .context import DATADIR
from typing import List
from model.common.commonmodel import CommonModel
from model.bcpnp.data import Bcpnp, General, JobOffer, ErAddress
from model.common.jobposition import Position
from model.common.rcic import Rcic
from model.common.advertisement import (
    Advertisement,
    Advertisements,
    InterviewRecord,
    InterviewRecords,
    RecruitmentSummary,
)
from model.common.person import Person, PersonalAssess
from model.common.address import Addresses
from model.common.wordmaker import WordMaker
import os


class Personal(Person):
    def __str__(self):
        return self.full_name


class EmployerTrainingModel(CommonModel):
    eraddress: List[ErAddress]
    general: General
    position: Position
    personal: Personal
    joboffer: JobOffer
    personalassess: PersonalAssess
    bcpnp: Bcpnp
    rcic: Rcic
    advertisement: List[Advertisement]
    interviewrecord: List[InterviewRecord]
    recruitmentsummary: RecruitmentSummary

    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self, excels=None, output_excel_file=None):
        if output_excel_file:
            excels = self.getExcels(
                [
                    "excel/er.xlsx",
                    "excel/pa.xlsx",
                    "excel/recruitment.xlsx",
                    "excel/bcpnp.xlsx",
                    "excel/rep.xlsx",
                ]
            )
        else:
            if excels is None and len(excels) == 0:
                raise ValueError(
                    "You must input excel file list as source data for validation"
                )
        # call parent class for validating
        super().__init__(excels, output_excel_file, globals())

    @property
    def work_location(self):
        addresses = Addresses(self.eraddress)
        return addresses.working

    @property
    def selected_contact(self):
        contacts = Contacts(self.contact)
        return contacts.preferredContact

    @property
    def summary(self):
        return InterviewRecords(self.interviewrecord)

    @property
    def advertisements(self):
        return Advertisements(self.advertisement)

    @property
    def person(self):
        return {
            "full_name": self.personal.full_name,
            "attributive": self.personal.attributive,
            "object": self.personal.object,
            "subject": self.personal.subject,
            "short_name": self.personal.short_name,
        }


class EmployerTrainingDocxAdaptor:
    def __init__(self, employer_training_obj: EmployerTrainingModel):
        self.employer_training_obj = employer_training_obj

    def re_generate_dict(self):
        summary_info = {
            "resume_num": self.employer_training_obj.summary.resume_num,
            "canadian_num": self.employer_training_obj.summary.canadian_num,
            "unknown_num": self.employer_training_obj.summary.unknown_num,
            "foreigner_num": self.employer_training_obj.summary.foreigner_num,
            "total_canadian": self.employer_training_obj.summary.total_canadian,
            "total_interviewed_canadians": self.employer_training_obj.summary.total_interviewed_canadians,
            "canadian_records": self.employer_training_obj.summary.canadian_records,
            "advertisement": self.employer_training_obj.advertisements,
            "personal": self.employer_training_obj.person,
            "date_of_offer": self.employer_training_obj.joboffer.date_of_offer,
            "work_start_date": self.employer_training_obj.joboffer.start_date_say,
            "joboffer_date": self.employer_training_obj.joboffer.date_of_offer,
            "work_location": self.employer_training_obj.work_location,
        }
        return {**self.employer_training_obj.dict(), **summary_info}

    def make(self, output_docx):
        template_path = os.path.abspath(
            os.path.join(DATADIR, "word/bcpnp_facts_brief_employer.docx")
        )
        wm = WordMaker(template_path, self.re_generate_dict(), output_docx)
        wm.make()



# from context import BASEDIR
from webform.prportal.i5562 import F5562
from webform.prportal.i5406 import F5406
from webform.prportal.i5669 import F5669
from webform.prportal.i0008 import F0008
from webform.prportal.login import Login
from webform.models.prmodel import PrModel
from webform.prportal.application import Application
import json, dotenv, os, argparse
from termcolor import colored
from docxtpl import DocxTemplate
import os,sys

# Get project's home directory, 
BASEDIR=os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
# All data directory
DATADIR=os.path.abspath(os.path.join(BASEDIR,'data'))


def login_prportal(rcic: str):
    # login
    path = os.path.abspath(os.path.join(os.path.expanduser("~"), ".prportalenv"))
    config = dotenv.dotenv_values(path)
    rcic_account = {
        "account": config.get(rcic + "_prportal_account"),
        "password": config.get(rcic + "_prportal_password"),
    }
    if not rcic_account["account"] or not rcic_account["password"]:
        print(
            colored(
                f"{rcic}'s prportal account and/or password is not existed. Check the .prportalenv file in your home directory and add your profile",
                "red",
            )
        )
        exit(1)

    rcic_obj = Login(rcic_account)
    return rcic_obj.login()


def output(args, actions):
    if args.to:
        with open(args.to, "w") as f:
            json.dump(actions, f, indent=3, default=str)
            print(colored(f"{args.to} is created", "green"))
    else:
        print(json.dumps(actions, indent=3, default=str))

def makeDocx(persons,docx_file):
    document=DocxTemplate(DATADIR+"/word/pr-appendix.docx")
    document.render(persons)
    document.save(docx_file)
    print(colored(f"{docx_file} saved","green"))


def main():
    parser = argparse.ArgumentParser(
        description="Used for generating data for webform filling."
    )
    # input source excel files for pa, sp, and dp
    parser.add_argument(
        "-pa",
        "--principal_applicant",
        help="Input principal applicant's excel file name",
    )
    parser.add_argument("-sp", "--spouse", help="Input spouse's excel file name")
    parser.add_argument(
        "-dp",
        "--dependants",
        help="Input dependants' excel file names, can be multiple.",
        nargs="+",
    )
    # get which rcic
    parser.add_argument("-r", "--rcic", help="Input rcic's name")
    # generate data for filling all, or specific form
    parser.add_argument(
        "-f",
        "--forms",
        help="Specify generating which form. Exp: -f 5669 5562 0008 5406. Without -f, the app will generate all forms' data ",
        nargs="+",
    )
    # generate 0008 pa's data
    parser.add_argument(
        "-f0008pa", "--f0008pa", help="Generate imm0008 pa's data", action="store_true"
    )
    # generate 0008 dps' data
    parser.add_argument(
        "-f0008dp", "--f0008dp", help="Generate imm0008 dps' data", action="store_true"
    )
    # save to file
    parser.add_argument("-t", "--to", help="Input the output json file name")
    
    # make an additional appendix docx file including all information
    parser.add_argument("-a", "--appendix", help="Input the appendix file name")
    
    args = parser.parse_args()

    # get pa, sp, and dps object
    if args.principal_applicant:
        pa = PrModel([args.principal_applicant])
    else:
        print(
            colored(
                "You didn't input principal applicant's excel file. Please use -pa excel.xlsl",
                "red",
            )
        )
        return

    if args.spouse:
        sp = PrModel([args.spouse])
    else:
        sp = None

    dps = []
    if args.dependants:
        for excel in args.dependants:
            dps.append(PrModel([excel]))

    # make appendix doc file
    if args.appendix:
        dps_above_18=[dp.dict() for dp in dps if dp.personal.age>=18]
        sp=[sp.dict()] if sp else []
        makeDocx({"persons":[pa.dict(),*sp,*dps_above_18]},args.appendix)
        return
    # actions container
    actions = []
    # login
    if not args.rcic:
        print(
            colored(
                "You did not speficy using which rcic's portal. Please use -r rcic name",
                "red",
            )
        )
        return
    actions += login_prportal(args.rcic)
    # determine create a new applicatoin or pick an existing application. 现在创建还是手工创建算了。
    # pick an application
    app = Application(pa)
    actions += app.pick()

    if args.f0008pa:
        f0008 = F0008(pa, sp, dps)
        actions += f0008.fill_pa()
        output(args, actions)
        return
    if args.f0008dp:
        f0008 = F0008(pa, sp, dps)
        actions += f0008.fill_dp()
        output(args, actions)
        return


    # if args.form exists, then loop the form and generate them, else generate all forms
    if args.forms:
        a5406 = a5562 = a5669 = a0008 = []
        for form in args.forms:
            match form:
                case "5406":
                    f5406 = F5406(pa, sp, dps)
                    a5406 = f5406.fill()
                case "5562":
                    f5562 = F5562(pa, sp, dps)
                    a5562 = f5562.fill()
                case "5669":
                    f5669 = F5669(pa, sp, dps)
                    a5669 = f5669.fill()
                case "0008":
                    f0008 = F0008(pa, sp, dps)
                    a0008 = f0008.fill()
                case _:
                    print(
                        colored(
                            f"{form} is not a valid form number in '5562','5406','5669','0008'",
                            "red",
                        )
                    )
                    return
        actions += a5406 + a5562 + a5669 + a0008
        output(args, actions)
        return
    else:
        f5406 = F5406(pa, sp, dps)
        actions += f5406.fill()
        f5562 = F5562(pa, sp, dps)
        actions += f5562.fill()
        f5669 = F5669(pa, sp, dps)
        actions += f5669.fill()
        f0008 = F0008(pa, sp, dps)
        actions += f0008.fill()
        output(args, actions)
        return

    
        

if __name__ == "__main__":
    main()



from pydantic import BaseModel,EmailStr,validator
from typing import Optional
from model.common.utils import normalize,trimString
from datetime import date,datetime
from dateutil.parser import parse,ParserError

    
class Person(BaseModel):
    last_name:str
    first_name:str
    sex:str
    dob:date
    citizen:str
    
    _normalize_first_name=validator('first_name',allow_reuse=True,check_fields=False)(normalize)
    _normalize_last_name=validator('last_name',allow_reuse=True,check_fields=False)(normalize)
    
    @property
    def full_name(self):
        return self.first_name+" "+self.last_name

    @property
    def age(self):
        the_day= date.today()
        try:
            return the_day.year - self.dob.year - ((the_day.month, the_day.day) < (self.dob.month, self.dob.day))
        except (ValueError,TypeError,ParserError) as err:
            print(err.args[0]," in file ",__file__)
    
    def ageOnDate(self,the_day= date.today()):
        if not isinstance(the_day,date):
            try:
                the_day=datetime.strptime(the_day,"%Y-%m-%d")
                return the_day.year - self.dob.year - ((the_day.month, the_day.day) < (self.dob.month, self.dob.day))
            except (ValueError,TypeError) as err:
                print(err.args[0]," in file ",__file__)
    
    # Third person subject 
    @property
    def subject(self):
        return "he" if self.sex.upper()=='M' or self.sex.upper()=="MALE" else 'she'
    
    # Third person object
    @property
    def object(self):
        return "him" if self.sex.upper()=='M' or self.sex.upper()=="MALE" else 'her'

    # third person attributive
    @property
    def attributive(self):
        return "his" if self.sex.upper()=='M' or self.sex.upper()=="MALE" else 'her'
    
    # salutation
    @property
    def salutation(self):
        return "Mr." if self.sex.upper()=='M' or self.sex.upper()=="MALE" else 'Ms.'
    
    @property
    def short_name(self):
        return self.salutation+' '+self.last_name

class PersonalAssess(BaseModel):
    work_experience_brief:str
    education_brief:str
    competency_brief:str
    language_brief:Optional[str]
    performance_remark:Optional[str]
    
    @property
    def why_qualified(self):
        qualifications=[self.work_experience_brief,self.education_brief,self.competency_brief,self.language_brief,self.performance_remark]
        return [q for q in qualifications if q is not None]
    
    @property
    def why_qualified_say(self):
        return ' '.join(self.why_qualified)

def main():
    p1={
        'first_name':'jacky',
        'last_name':"zhang",
        'dob':"1969-02-11",
        'sex':"Male",
        'citizen':"China"
    }
    jacky=Person(**p1)
    print(jacky.full_name,jacky.age,jacky.ageOnDate('2056-02-11'))

if __name__=="__main__":
    main()
    


from pydantic import BaseModel
from typing import List,Optional
from datetime import date

class FamilyBase(BaseModel):
    last_name:str
    first_name:str	
    native_last_name:Optional[str]	
    native_first_name:Optional[str] 
    date_of_birth:date  
    date_of_death:Optional[date]
    place_of_birth:str
    birth_country:str 
    relationship:str
    

class FamilyMembers():
    
    def __init__(self,members:List[FamilyBase]):
        self.members=members
        
    @property
    def spouse(self):
        s=[ s for s in self.members if s.relationship=='Spouse']
        return s[0] if len(s)==1 else None
    
    @property
    def siblings(self):
        s=[ s for s in self.members if s.relationship in ['Brother','Sister']]
        return s
    
    @property
    def dependants(self):
        s=[ s for s in self.members if s.relationship in ['Son','Daughter']]
        return s
    
    @property
    def father(self):
        s=[ s for s in self.members if s.relationship == 'Father']
        return s[0] if len(s)==1 else None
    
    @property
    def mother(self):
        s=[ s for s in self.members if s.relationship == 'Mother']
        return s[0] if len(s)==1 else None
    
    @property
    def parents(self):
        s=[ s for s in self.members if s.relationship in  ['Father','Mother']]
        return s




from webform.models.definition import Action, Role
from webform.models.prmodel import PrModel
from model.common.address import Addresses
from typing import List
from webform.models.prmodel import Family


class F5406:
    # map of input and output marital status
    marital_map = {
        "Annulled Marriage": "1: 09",
        "Common-Law": "2: 03",
        "Divorced": "3: 04",
        "Married": "5: 01",
        "Separated": "4: 05",
        "Single": "7: 02",
        "Unknown": "9: 00",
        "Widowed": "8: 06",
    }
    # dict of field ids
    fild_id = {
        "pa": {
            "fn": "#applicantFullName",
            "dob": "#applicantDOB",
            "birth_place": "#applicantBirthplace",
            "marital_status": "#applicantMaritalStatus",
            "email": "#applicantEmail",
            "address": "#applicantAddress",
        },
        "sp": {
            "fn": "#partnerFullName",
            "dob": "#partnerDOB",
            "birth_place": "#partnerBirthplace",
            "marital_status": "#partnerMaritalStatus",
            "email": "#partnerEmail",
            "address": "#partnerAddress",
        },
        "mom": {
            "fn": "#motherFullName",
            "dob": "#motherDOB",
            "birth_place": "#motherBirthplace",
            "marital_status": "#motherMaritalStatus",
            "email": "#motherEmail",
            "address": "#motherAddress",
        },
        "dad": {
            "fn": "#fatherFullName",
            "dob": "#fatherDOB",
            "birth_place": "#fatherBirthplace",
            "marital_status": "#fatherMaritalStatus",
            "email": "#fatherEmail",
            "address": "#fatherAddress",
        },
    }

    def __init__(self, pa: PrModel, sp: PrModel | None, dps: List[PrModel]):
        self.pa = pa
        self.sp = sp
        self.dps = dps

    @property
    def pdf(self):
        return [{"action_type": Action.Pdf.value}]

    def goTo5406(self):
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Start/Edit for 5406 form",
                "actions": [],
                "id": "body > pra-root > pra-localized-app > main > div > pra-intake-landing-page > pra-web-form-table > div > table > tbody > tr:nth-child(13) > td:nth-child(5) > button",
            }
        ]

    def add_family_member(self):
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Add new family member",
                "actions": [],
                "next_page_tag":"#fatherAddress",
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm5406 > pra-imm5406-start > pra-family-table > button",
            }
        ]

    def pick_whom(self, role):
        return [
            {
                "action_type": Action.Radio.value,
                "label": "Indicate whether you are filling out this form for",
                "id": "#principalAppYes" if role == Role.PA else "#principalAppNo",
            }
        ]

    def completeReturn(self):
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Complete and return to application",
                "actions": [],
                "next_page_tag":"body > pra-root > pra-localized-app > main > div > pra-intake-landing-page > div.intake-landing-page_submit-application.ng-star-inserted > button",
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm5406 > lib-navigation-buttons > div > button.btn.btn-primary",
            }
        ]

    def sectionABlock(
        self, relationship, person: PrModel = None, family_member: Family = None
    ):
        return [
            {
                "action_type": "Input",
                "label": "Full name",
                "value": person.personal.first_name + " " + person.personal.last_name +" / "+person.personal.native_first_name+" "+person.personal.native_last_name
                if person
                else family_member.first_name + " " + family_member.last_name+" / "+family_member.native_first_name+"  "+family_member.native_last_name,
                "id": F5406.fild_id[relationship]["fn"],
                "length": 63,
                "required": True,
            },
            {
                "action_type": "Input",
                "label": "Date of birth",
                "value": person.personal.dob.strftime("%Y/%m/%d")
                if person
                else family_member.date_of_birth.strftime("%Y/%m/%d"),
                "id": F5406.fild_id[relationship]["dob"],
                "length": 11,
                "required": True,
            },
            {
                "action_type": "Input",
                "label": "Country or territory of birth",
                "value": person.personal.country_of_birth
                + "/"
                + person.personal.place_of_birth
                if person
                else family_member.birth_country + "/" + family_member.place_of_birth,
                "id": F5406.fild_id[relationship]["birth_place"],
                "length": 30,
                "required": True,
            },
            {
                "action_type": "Select",
                "label": "Marital status",
                "value": F5406.marital_map[person.marriage.marital_status]
                if person
                else F5406.marital_map[family_member.marital_status],
                "id": F5406.fild_id[relationship]["marital_status"],
            },
            {
                "action_type": "Input",
                "label": "Email",
                "value": person.personal.email if person else family_member.email,
                "id": F5406.fild_id[relationship]["email"],
                "length": 40,
                "required": False,
            },
            {
                "action_type": "Input",
                "label": "Address",
                "value": Addresses(person.address).residential
                if person
                else family_member.address,
                "id": F5406.fild_id[relationship]["address"],
                "length": 80,
                "required": True,
            },
        ]

    def makeSectionB(self, children: List[Family]):
        value = []

        for index, child in enumerate(children):
            block = [
                {
                    "action_type": Action.Input.value,
                    "label": "Relationship",
                    "id": "#relationship" + str(index),
                    "value": child.relationship,
                    "length": 27,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Full name",
                    "id": "#fullName" + str(index),
                    "value": child.first_name + " " + child.last_name+" / "+child.native_first_name+" "+child.native_last_name,
                    "length": 63,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Date of birth",
                    "id": "#dob" + str(index),
                    "value": child.date_of_birth.strftime("%Y/%m/%d"),
                    "length": 10,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Country",
                    "id": "#countryOfBirth" + str(index),
                    "value": child.birth_country + "/" + child.place_of_birth,
                    "length": 30,
                    "required": True,
                },
                {
                    "action_type": Action.Select.value,
                    "label": "Maritial status",
                    "id": "#maritalStatus" + str(index),
                    "value": F5406.marital_map[child.marital_status],
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Email",
                    "id": "#emailAddress" + str(index),
                    "value": child.email,
                    "length": 40,
                    "required": False,
                },
                {
                    "action_type": Action.Areatext.value,
                    "label": "Address",
                    "id": "#address" + str(index),
                    "value": child.address,
                    "length": 80,
                    "required": True,
                },
            ]
            value.append(block)

        return [
            {
                "action_type": Action.RepeatSection.value,
                "button_text": "Add another",
                "value": value,
            }
        ]

    def makeSectionC(self, siblings: List[Family]):
        value = []

        for index, sibling in enumerate(siblings):
            block = [
                {
                    "action_type": Action.Input.value,
                    "label": "Relationship",
                    "id": "#relationship" + str(index),
                    "value": sibling.relationship,
                    "length": 27,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Full name",
                    "id": "#fullName" + str(index),
                    "value": sibling.first_name + " " + sibling.last_name+" /  "+sibling.native_first_name+" "+sibling.native_last_name,
                    "length": 63,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Date of birth",
                    "id": "#dob" + str(index),
                    "value": sibling.date_of_birth.strftime("%Y/%m/%d"),
                    "length": 10,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Country",
                    "id": "#countryOfBirth" + str(index),
                    "value": sibling.birth_country + "/" + sibling.place_of_birth,
                    "length": 30,
                    "required": True,
                },
                {
                    "action_type": Action.Select.value,
                    "label": "Maritial status",
                    "id": "#maritalStatus" + str(index),
                    "value": F5406.marital_map[sibling.marital_status],
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Email",
                    "id": "#emailAddress" + str(index),
                    "value": sibling.email,
                    "length": 40,
                    "required": False,
                },
                {
                    "action_type": Action.Areatext.value,
                    "label": "Address",
                    "id": "#address" + str(index),
                    "value": sibling.address,
                    "length": 80,
                    "required": True,
                },
            ]
            value.append(block)

        return [
            {
                "action_type": Action.RepeatSection.value,
                "button_text": "Add another",
                "value": value,
            }
        ] if len(siblings)>0 else []

    def sectionA(self, person: PrModel, role):
        applicant = self.sectionABlock("pa", person=person)

        # spouse = self.sectionABlock("sp", self.sp) if self.sp else []
        spouse_obj=[person for person in person.family if person.relationship.lower()=="spouse"]
        mother_obj = [
            person
            for person in person.family
            if person.relationship.lower() == "mother"
        ]
        father_obj = [
            person
            for person in person.family
            if person.relationship.lower() == "father"
        ]
        spouse=self.sectionABlock("sp",family_member=spouse_obj[0]) if spouse_obj else []
        mother = (
            self.sectionABlock("mom", family_member=mother_obj[0]) if mother_obj else []
        )
        father = (
            self.sectionABlock("dad", family_member=father_obj[0]) if father_obj else []
        )
        whom = []
        match role:
            case Role.PA:
                whom = self.pick_whom(Role.PA)
            case Role.SP:
                whom = self.pick_whom(Role.SP)
            case Role.DP:
                whom = self.pick_whom(Role.DP)
        actions = whom + applicant + spouse + mother + father + self.pdf
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Imm5406 additional family information Section A",
                "actions": actions,
                "next_page_tag":"#address0",
                "screen_shoot":{
                    "format":"Pdf",
                    "wait_for":"#fatherAddress"
                },
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm5406 > lib-navigation-buttons > div > button.btn.btn-primary",
            }
        ]

    def sectionB(self, person: PrModel):
        children = [p for p in person.family if p.relationship in ["Daughter", "Son"]]
        actions = self.makeSectionB(children) + self.pdf
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Imm5406 additional family information Section B: Children",
                "actions": actions,
                "next_page_tag":"#emailAddress0",
                "screen_shoot":{
                    "format":"Pdf",
                    "wait_for":"#address0"
                },
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm5406 > lib-navigation-buttons > div > button.btn.btn-primary",
            }
        ]

    def sectionC(self, person: PrModel):
        siblings = [p for p in person.family if p.relationship in ["Brother", "Sister"]]
        actions = self.makeSectionC(siblings) + self.pdf
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Imm5406 additional family information Section C: Siblings",
                "actions": actions,
                "next_page_tag":"body > pra-root > pra-localized-app > main > div > pra-imm5406 > lib-navigation-buttons > div > button.btn.btn-primary",
                "screen_shoot":{
                    "format":"Pdf",
                    "wait_for":"#address1"
                },
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm5406 > lib-navigation-buttons > div > button.btn.btn-primary",
            }
        ]

    def fill(self):
        pa_form = (
            self.add_family_member()
            + self.sectionA(self.pa, role=Role.PA)
            + self.sectionB(self.pa)
            + self.sectionC(self.pa)
        )
        sp_form = (
            self.add_family_member()
            + self.sectionA(self.sp, role=Role.SP)
            + self.sectionB(self.sp)
            + self.sectionC(self.sp)
            if self.sp
            else []
        )
        dp_forms = []
        for dp in self.dps:
            if dp.personal.age >= 18:
                dp_forms += (
                    self.add_family_member()
                    + self.sectionA(dp, role=Role.DP)
                    + self.sectionB(dp)
                    + self.sectionC(dp)
                )

        return self.goTo5406() + pa_form + sp_form + dp_forms + self.completeReturn()



from .context import DATADIR
from dataclasses import dataclass
from typing import List
from model.common.commonmodel import CommonModel
from model.lmia.data import (
    LmiaCase,
    Finance,
    Lmi,
    Emp5626,
    Emp5627,
    Emp5593,
    General,
    JobOffer,
    PersonalAssess,
    Rcic,
)
from model.common.jobposition import Position
from model.common.advertisement import InterviewRecord, Advertisement
from model.common.person import Person
from model.common.wordmaker import WordMaker
import os


class Personal(Person):
    def __str__(self):
        return self.full_name


class EmployerTrainingModel(CommonModel):
    general: General
    position: Position
    personal: Personal
    joboffer: JobOffer
    personalassess: PersonalAssess
    lmiacase: LmiaCase
    finance: List[Finance]
    lmi: Lmi
    rcic: Rcic
    advertisement: List[Advertisement]
    interviewrecord: List[InterviewRecord]

    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self, excels=None, output_excel_file=None):
        if output_excel_file:
            excels = self.getExcels(
                [
                    "excel/er.xlsx",
                    "excel/pa.xlsx",
                    "excel/recruitment.xlsx",
                    "excel/lmia.xlsx",
                    "excel/rep.xlsx",
                ]
            )
        else:
            if excels is None and len(excels) == 0:
                raise ValueError(
                    "You must input excel file list as source data for validation"
                )
        # call parent class for validating
        super().__init__(excels, output_excel_file, globals())

    def makeDocx(self, output_docx):
        template_path = os.path.abspath(
            os.path.join(DATADIR, "word/lmia_employer_training.docx")
        )
        wm = WordMaker(template_path, self, output_docx)
        wm.make()


class M5593EmployerTrainingModel(EmployerTrainingModel):
    pass


class M5626EmployerTrainingModel(EmployerTrainingModel):
    pass


class M5627EmployerTrainingModel(EmployerTrainingModel):
    pass



from setuptools import setup
from setuptools import find_packages, find_namespace_packages

# Load the README file.
with open(file="README.md", mode="r") as readme_handle:
    long_description = readme_handle.read()

setup(
    # Define the library name, this is what is used along with `pip install`.
    name="imm-apps",
    # Define the author of the repository.
    author="Jacky Zhang",
    # Define the Author's email, so people know who to reach out to.
    author_email="jackyzhang1969@gmail.com",
    # Define the version of this library.
    # Read this as
    #   - MAJOR VERSION 0
    #   - MINOR VERSION 1
    #   - MAINTENANCE VERSION 0
    version="1.0.13",
    # Here is a small description of the library. This appears
    # when someone searches for the library on https://pypi.org/search.
    description="A python client library used to describe program model and get validated data.",
    # I have a long description but that will just be my README
    # file, note the variable up above where I read the file.
    long_description=long_description,
    # This will specify that the long description is MARKDOWN.
    long_description_content_type="text/markdown",
    # Here is the URL where you can find the code, in this case on GitHub.
    url="",
    # These are the dependencies the library needs in order to run.
    install_requires=[],
    # Here are the keywords of my library.
    keywords="Canada, Immigration,Case, Programs, APIs, Assess,web,form,programtically filling",
    # here are the packages I want "build."
    packages=find_packages(),
    # here we specify any package data.
    package_data={
        # And include any files found subdirectory of the "td" package.
        "data": ["excel/*.xlsx", "word/*.docx", "forms/*.pdf"],
    },
    # I also have some package data, like photos and JSON files, so
    # I want to include those as well.
    include_package_data=True,
    # Here I can specify the python version necessary to run this library.
    python_requires=">=3.10",
    # Additional classifiers that give some characteristics about the package.
    # For a complete list go to https://pypi.org/classifiers/.
    classifiers=[
        # I can say what phase of development my library is in.
        "Development Status :: 3 - Alpha",
        # Here I'll add the audience this library is intended for.
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Financial and Insurance Industry",
        # Here I'll define the license that guides my library.
        "License :: OSI Approved :: MIT License",
        # Here I'll note that package was written in English.
        "Natural Language :: English",
        # Here I'll note that any operating system can use it.
        "Operating System :: OS Independent",
        # Here I'll specify the version of Python it uses.
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        # Here are the topics that my library covers.
        # 'Topic :: Data',
        # 'Topic :: Immigration',
        # 'Topic :: Canada'
    ],
    # Here defines the console scripts. Terminal command, and it's source.
    entry_points={
        "console_scripts": [
            "ds=bin.datasource:main",  # ds是命令行命令， source.source:main是对应的函数所在包中的位置
            "prd=bin.prdata:main",
            "bcd=bin.bcdata:main",
            "bcpnp=bin.bcpnp:main",
            "excel=bin.excel:main",
            "exp=bin.experience:main",
            "lmia=bin.lmia:main",
            "pr=bin.pr:main",
            "qa=bin.quickassess:main",
            "recruit=bin.recruit:main",
            "trv=bin.trv:main",
            "pdf=utils.pdf:main",
            "files=utils.files:main",
            "clb=bin.language:main",
            "pdff=bin.pdffiller:main",
        ]
    },
)



from webform.models.definition import Action
from abc import ABC, abstractclassmethod


class Dashboard:
    @abstractclassmethod
    def jump(self, page: str):
        pass

    @property
    def save(self):
        return [
            {
                "action_type": Action.Button.value,
                "label": "Save",
                "id": "#tabset-navigation > div > div > div.form-group.pull-right > uf-save-button > button",
            }
        ]


class DashboardReg(Dashboard):
    def jump(self, page: str):
        pages = {
            "Registrant": "object:135",
            "Education": "object:136",
            "Work Experience": "object:137",
            "Job Offer": "object:138",
            "Language": "object:139",
            "Submit": "object:140",
        }
        return [
            {
                "action_type": Action.Select.value,
                "label": "Jump to tab",
                "id": "#navigate-to",
                "value": pages[page],
            }
        ]


class DashboardApp(Dashboard):
    def jump(self, page: str):
        pages = {
            "Applicant": "object:224",
            "Education": "object:225",
            "Work Experience": "object:226",
            "Family": "object:227",
            "Job Offer": "object:228",
            "Attachments": "object:230",
            "Submit": "object:231",
        }
        return [
            {
                "action_type": Action.Select.value,
                "label": "Jump to tab",
                "id": "#navigate-to",
                "value": pages[page],
            }
        ]



from webform.models.definition import Action
from webform.models.lmiaprmodel import LMIAPrModel,LMIAHWSModel
from typing import Union
import base64

# This step includs actions of signing in, picking Skill Immigration automatically, select and confirm which stream to apply through
class Login:
    def __init__(
        self,
        lmia: Union[LMIAPrModel,LMIAHWSModel],
    ):
        self.lmia = lmia

    def encode(self, password):
        password_bytes = password.encode("ascii")
        base64_bytes = base64.b64encode(password_bytes)
        return base64_bytes.decode("ascii")

    def login(self,user_id,password,security_answers:object ):
        login_actions= [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Log in",
                "actions": [
                    {
                        "action_type": Action.GotoPage.value,
                        "url": "https://tfwp-jb.lmia.esdc.gc.ca/employer/",
                    },
                    {
                        "action_type": Action.Login.value,
                        "label": "Login",
                        "account": user_id,
                        "password": self.encode(password),
                        "account_element_id": "#loginForm\\:input-email",
                        "password_element_id": "#loginForm\\:input-password",
                        "login_button_id": "#loginForm\\:j_id_43",
                    },
                ],
                "id": None,  #
            }
        ]
        security_actions=[
            {
                "action_type": Action.WebPage.value,
                "page_name": "Security check",
                "actions": [
                    {
                        "action_type":Action.Security.value,
                        "portal":"lmiaportal",
                        "question_element_id":"#securityForm > fieldset > div > p",
                        "answer_element_id": "#securityForm\\:input-security-answer",
                        "continue_element_id": "#continueButton",
                        "security_answers":security_answers
                    }],
                "id":None,
            },
            {
                "action_type": Action.WebPage.value,
                "page_name": "Important message",
                "actions": [],
                "id":"#modal-accept"
            }
            ]

        return login_actions+security_actions



# import context
from model.pr.m0008 import M0008Model
from model.pr.m5406 import M5406Model
from model.pr.m5562 import M5562Model
from model.pr.m5669 import M5669Model
import argparse,os
from termcolor import colored

def main():
    parser=argparse.ArgumentParser(description="used for processing permanent resident visa")
    parser.add_argument("-p", "--program", help="input program code: 0008 5669 5562 5406 ")
    parser.add_argument("-t", "--to", help="input excel file name for output")
    args = parser.parse_args()
    
    valid_programs={
        '0008':M0008Model,
        '5406':M5406Model,
        '5562':M5562Model,
        '5669':M5669Model
        }
    if args.program not in valid_programs.keys():
        print(colored(f'Your input program code {args.program} is not valid. ','red'))
        return
    
    if args.to: # and not os.path.isfile(args.to):
        valid_programs[args.program](output_excel_file=args.to)
        return 

    print(colored('Invalid command and flag combination... try pr -h for help','red'))
    
    
if __name__=='__main__':
    main()


from pydantic import BaseModel, EmailStr
from typing import Optional,List
from datetime import date

from model.common.commonmodel import CommonModel

class Family(BaseModel):
    relationship:Optional[str]
    last_name:Optional[str]	
    first_name:Optional[str]	
    native_last_name:Optional[str]	
    native_first_name:Optional[str]
    date_of_birth:Optional[date]	
    date_of_death:Optional[date]
    place_of_birth:Optional[str]
    birth_country:Optional[str]	
    marital_status:Optional[str]	
    email:Optional[EmailStr]	
    address:Optional[str]

class M5406Model(CommonModel):
    family:List[Family]
    
    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self,excels=None,output_excel_file=None):
        if output_excel_file:
            excels=self.getExcels(['excel/pa.xlsx'])
        else:
            if excels is None and len(excels)==0:
                raise ValueError('You must input excel file list as source data for validation')
        super().__init__(excels,output_excel_file,globals())


# from context import BASEDIR
from webform.bcpnp.register import Register
from webform.models.bcpnpmodel_app import BcpnpEEModelApp, BcpnpModelApp
from webform.models.bcpnpmodel_reg import BcpnpModelReg, BcpnpEEModelReg
from webform.bcpnp.login import Login
from webform.bcpnp.registrant import Registrant
from webform.bcpnp.education_reg import Education
from webform.bcpnp.employment import Employment
from webform.bcpnp.joboffer_reg import Joboffer
from webform.bcpnp.language import Language
from webform.bcpnp.submit import Submit
from webform.bcpnp.applicant import Applicant
from webform.bcpnp.education_app import EducationApp
from webform.bcpnp.workexperience import WorkExperience
from webform.bcpnp.family import Family
from webform.bcpnp.joboffer_app import JobofferApp


import json, argparse, os
from termcolor import colored


def save(actions, filename):
    with open(filename, "w") as fp:
        json.dump(actions, fp, indent=3, default=str)
    print(colored(f"{filename} has been saved.", "green"))


def main():
    parser = argparse.ArgumentParser(
        description="used for generating bcpnp form filling data"
    )
    # commands
    parser.add_argument("-e", "--excel", help="input bcpnp excel file. Will create one if the excel issn't exsited")
    parser.add_argument(
        "-t", "--task", help="tasks for p: profile, r: register a: application"
    )
    parser.add_argument("-j", "--json", help="output json file name")

    # flags
    parser.add_argument(
        "-i",
        "--initial",
        help="indicator stands for it is initial time for creating a stream to fill",
        action="store_true",
    )
    parser.add_argument(
        "-ee",
        "--express_entry",
        help="indicator stnads for it is Enpress Entry or not",
        action="store_true",
    )

    args = parser.parse_args()

    if not args.task:
        print(
            colored(
                "You must indicate which task to do by type -t plus p, r, or a. ", "red"
            )
        )
        return
    
    if not args.excel:
        print(
            colored(
                "You must indicate which task to do by type -t plus p, r, or a. ", "red"
            )
        )
        return
    
    if args.excel and not os.path.exists(args.excel):
        print(colored(f"{args.excel} is not existed. ", "red"))
        match args.task:
            case "p" | "r":
                client = (
                BcpnpEEModelReg(output_excel_file=args.excel)
                if args.express_entry
                else BcpnpModelReg(output_excel_file=args.excel)
                )
                print(colored(f"{args.excel} has been created. ", "green"))
            case "a":
                client = (
                BcpnpEEModelApp(output_excel_file=args.excel)
                if args.express_entry
                else BcpnpModelApp(output_excel_file=args.excel)
                )
                print(colored(f"{args.excel} has been created. ", "green"))
        return
    
    # actions container
    actions = []

    # excute tasks
    match args.task:
        case "p":  # profile
            client = (
                BcpnpEEModelReg(excels=[args.excel])
                if args.express_entry
                else BcpnpModelReg(excels=[args.excel])
            )
            # register
            register = Register(client)
            actions += register.register()
        case "r":  # registration
            client = (
                BcpnpEEModelReg(excels=[args.excel])
                if args.express_entry
                else BcpnpModelReg(excels=[args.excel])
            )
            # signing in, pick skill immigration, select stream and confirm
            actions += Login(client).login(initial=True if args.initial else False)
            # Registrant
            actions += Registrant(client).fill()
            # Education
            actions += Education(client).fill()
            # Employment
            actions += Employment(client).fill()
            # Job offer
            actions += Joboffer(client).fill()
            # Language
            actions += Language(client).fill()
            # Submit
            actions += Submit(client).fill()
        case "a":  # application
            client = (
                BcpnpEEModelApp(excels=[args.excel])
                if args.express_entry
                else BcpnpModelApp(excels=[args.excel])
            )
            # login
            # signing in, pick skill immigration, select stream and confirm
            actions += Login(client).login(initial=False)
            # Applicant
            actions += Applicant(client).fill()
            # Education
            actions += EducationApp(client).fill()
            # Employment
            actions += WorkExperience(client).fill()
            # Family
            actions += Family(client).fill()
            # Job offer
            actions += JobofferApp(client).fill()
            # Submit
            actions += Submit(client).fill()
        case _:  # profile
            print(
                colored(
                    f"{args.task} is wrong indicator. There are 3 types, in which p stands for profile, r for register, and a for application.",
                    "reg",
                )
            )
            return
    save(actions, args.json) if args.json else print(
        json.dumps(actions, indent=3, default=str)
    )


if __name__ == "__main__":
    main()



from docxtpl import DocxTemplate
from datetime import datetime


class WordTpl():

    def __init__(self,template,context,output):
       self.document=DocxTemplate(template)
       self.context=context
       self.output=output

    def render(self):
        self.document.render(self.context)
        self.document.save(self.output) 

context={
    "provider":{
        "name":"Jacky Zhang",
        "rcicNum":"R511623",
        "email":"noah.consultant@outlook.com",
        "phone":"7783215110",
        "address":"50-3306 Princeton Ave, Coquitlam, BC V3E 0M9"
    },
     "receiver":{
        "name":"Lily Zhao",
        "rcicNum":"R512623",
        "email":"lily_zhao@outlook.com",
        "phone":"7784445110",
        "address":"51-3306 Princeton Ave, Coquitlam, BC V3E 0M9"
    },
    "service":"BCPNP application",
    "client":"Liu Xiaoming",
    "price":5000,
    "gst":0.05,
    "terms":[
        {"subPrice":2500,"payDay":"the day of signing agreement"},
        {"subPrice":2500,"payDay":"the day of submission of the application"}
    ],
    "date":datetime.today().strftime("%Y-%m-%d")
}
def main():
    word=WordTpl('/Users/jacky/tools/templates/word/co-counsel agreement.docx',context,'lynn.docx')
    word.render()

if __name__=="__main__":
    main()




phone_type={
    'residential':'01',
    "cellular":"02",
    "business":"03"
}

tr_canada_status={
    "citizen":'01',
    'permanent resident':'02',
    'visitor':'03',
    'worker':'04',
    'student':'05',
    'other':'06',
    'protected person':'07',
    'refugee claimant':'08',
    'foreign national':'09'
}

# 编号有问题。 处理married和common law是对的。 其他不确定。
tr_marital_status={
    "annulled marriage":'09',
    'common-law':'03',
    'divorced':'04',
    'legally separated':'05',
    'married':'01',
    'single':'02',
    'unknown':'00',
    'widowed':'06'
}

marital_stauts_5645={
    "annulled marriage":'1',
    'common-law':'2',
    'divorced':'3',
    'legally separated':'4',
    'married':'5',
    'single':'6',
    'widowed':'7'
}

bool_convert={
    True: 'Y',
    False: "N",
    None:'N'
}
#TODO: 可以考虑分类，加拿大放一个特别类别

tr_portal_ca_province={
    "Newfoundland and Labrador": "01",
    "Prince Edward Island": "02",
    "Nova Scotia": "03",
    "New Brunswick": "04",
    "Quebec": "05",
    "Ontario": "06",
    "Manitoba": "07",
    "Saskatchewan": "08",
    "Alberta": "09",
    "British Columbia": "11",
    "Northwest Territories": "10",
    "Yukon": "12",
    "Nunavut": "64"
}
tr_portal_residence_status={
    "Citizen": "01",
    "Foreign National": "09",
    "Other": "06",
    "Permanent resident": "02",
    "Protected Person": "07",
    "Refugee Claimant": "08",
    "Student": "05",
    "Visitor": "03",
    "Worker": "04"
}

tr_portal_marital_status=[
    "Annulled Marriage",
    "Common Law",
    "Divorced",
    "Married",
    "Separated",
    "Single",
    "Widowed"
]

# PDF forms for imm1294 and imm5709 study format
tr_form_study_level={
    "Primary School":'1',
    "Secondary School":'2',
    "PTC/TCST/DVS/AVS":'10',
    "CEGEP - Pre-university":"11",
    "CEGEP - Technical":"12",
    "College - Certificate":"13",
    "College - Diploma":"14",
    "College - Applied degree":"15",
    "University - Bachelor's Deg.":"04",
    "University - Master's Deg.":"05",
    "University - Doctorate":"06",
    "University - Other Studies":"07",
    "ESL/FSL":"16",
    "ESL/FSL and College":"17",
    "ESL/FSL and University":"18",
    "Other Studies":"08",
    "Not Applicable":"19"
}

tr_form_study_field={
    "Arts/Humanities/Social Science":"01",
    "Arts, Fine/Visual/Performing":"02",
    "Business/Commerce":"03",
    "Computing/IT":"04",
    "ESL/FSL":"05",
    "Flight Training":"06",
    "Hospitality/Tourism":"07",
    "Law":"08",
    "Medicine":"09",
    "Science, Applied":"10",
    "Sciences, General":"11",
    "Sciences, Health":"12",
    "Trades/Vocational":"13",
    "Theology/Religious Studies":"14",
    "Other":"15",
    "Agric/Agric Ops/Rel Sciences":"16",
    "Architecture and Rel Services":"17",
    "Biological/Biomed Sciences":"18",
    "Business/Mgmt/Marketing":"19"
}

# tr portal study format
tr_portal_study_level={
    "Primary School": "01",
    "Secondary School": "02",
    "PTC/TCST/DVS/AVS": "10",
    "CEGEP - Pre-university": "11",
    "CEGEP - Technical": "12",
    "College - Certificate": "13",
    "College - Diploma": "14",
    "College - Applied degree": "15",
    "University - Bachelor's Deg.": "04",
    "University - Master's Deg.": "05",
    "University - Doctorate": "06",
    "University - Other Studies": "07",
    "ESL/FSL": "16",
    "ESL/FSL and College": "17",
    "ESL/FSL and University": "18",
    "Other Studies": "08",
    "Not Applicable": "19"
}
tr_portal_study_field={
    "Agric/Agric Ops/Rel Sciences": "16",
    "Architecture and Rel Services": "17",
    "Arts, Fine/Visual/Performing": "02",
    "Arts/Humanities/Social Science": "01",
    "Biological/Biomed Sciences": "18",
    "Business/Commerce": "03",
    "Business/Mgmt/Marketing": "19",
    "Computing/IT": "04",
    "ESL/FSL": "05",
    "Flight Training": "06",
    "Hospitality/Tourism": "07",
    "Law": "08",
    "Medicine": "09",
    "Other": "15",
    "Science, Applied": "10",
    "Sciences, General": "11",
    "Sciences, Health": "12",
    "Theology/Religious Studies": "14",
    "Trades/Vocational": "13"
}


from webform.bcpnp.dashboard import DashboardApp, DashboardReg
from webform.models.definition import Action
from webform.models.bcpnpmodel_app import BcpnpModelApp, BcpnpEEModelApp
from model.common.address import Addresses
from model.common.contact import Contacts
from webform.bcpnp.data import country_map, getLegalStructure, getCompanyIndustry
from typing import Union
from model.common.jobofferbase import JobofferBase


class JobofferApp:
    def __init__(self, person: Union[BcpnpModelApp, BcpnpEEModelApp]):
        self.person = person

    def fill(self):
        legal_other = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "If other, specify",
                    "id": "#BCPNP_App_Emp_Comp_LegalStructure_Other",
                    "value": self.person.general.other_explaination,
                    "required": True,
                    "length": 100,
                }
            ]
            if getLegalStructure(self.person.general.corporate_structure, "BC")
            == "Other"
            else []
        )
        legal_structure = [
            {
                "action_type": Action.Select.value,
                "label": "Company legal structure",
                "id": "#BCPNP_App_Emp_Comp_LegalStructure",
                "value": getLegalStructure(
                    self.person.general.corporate_structure, "BC"
                ),
            }
        ] + legal_other

        company_details = [
            {
                "action_type": Action.Input.value,
                "label": "Company legal name",
                "id": "#syncA_App_Emp_Comp_LegalName",
                "value": self.person.general.legal_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Company operating name",
                "id": "#syncA_App_Emp_Comp_OperName",
                "value": self.person.general.operating_name,
                "required": False,
                "length": 100,
            },
            *legal_structure,
            {
                "action_type": Action.Input.value,
                "label": "Incorporation or registration number",
                "id": "#syncA_App_Emp_IncorpNo",
                "value": self.person.general.registration_number,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Number of full-time equivalent employees",
                "id": "#syncA_App_Emp_EmployeesNo",
                "value": str(self.person.general.ft_employee_number),
                "required": True,
                "length": 10,
            },
            {
                "action_type": Action.Input.value,
                "label": "Year established in B.C.",
                "id": "#BCPNP_App_Emp_YearBC",
                "value": self.person.general.establish_date,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Select.value,
                "label": "Company economic sector",
                "id": "#syncA_App_Emp_Sector",
                "value": getCompanyIndustry(self.person.general.industry),
            },
            {
                "action_type": Action.Input.value,
                "label": "Company website",
                "id": "#BCPNP_App_Emp_Website",
                "value": self.person.general.website,
                "required": False,
                "length": 100,
            },
        ]
        business_address = Addresses(self.person.eraddress).business
        business_location = [
            {
                "action_type": Action.Input.value,
                "label": "Unit number",
                "id": "#BCPNP_App_Emp_CompAddrUnit",
                "value": business_address.unit,
                "required": False,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Street address",
                "id": "#BCPNP_App_Emp_BusAddr",
                "value": business_address.street_address,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "City/Town",
                "id": "#syncA_App_Emp_BusCity",
                "value": business_address.city,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Province",
                "id": "#BCPNP_App_Emp_BusProvince",
                "value": business_address.province,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Postal code",
                "id": "#syncA_App_Emp_BusPostal",
                "value": business_address.post_code,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Country",
                "id": "#BCPNP_App_Emp_BusCountry",
                "value": country_map[business_address.country],
                "required": True,
                "length": 100,
            },
        ]
        mailing_address = Addresses(self.person.eraddress).mailing
        is_same = [
            {
                "action_type": Action.Radio.value,
                "label": "Is the mailing address the same as the physical address? ",
                "id": "#BCPNP_App_MailAddrSame-Yes"
                if business_address == mailing_address
                else "#BCPNP_App_MailAddrSame-No",
            }
        ]
        mailing_location = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Unit number",
                    "id": "#BCPNP_App_Emp_CompAltAddrUnit",
                    "value": mailing_address.unit,
                    "required": False,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Street address",
                    "id": "#BCPNP_App_Emp_MailAddr",
                    "value": mailing_address.street_address,
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "City/Town",
                    "id": "#syncA_App_Emp_MailCity",
                    "value": mailing_address.city,
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Province",
                    "id": "#BCPNP_App_Emp_MailProvince",
                    "value": mailing_address.province,
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Postal code",
                    "id": "#BCPNP_App_Emp_MailPostal",
                    "value": mailing_address.post_code,
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Country",
                    "id": "#BCPNP_App_Emp_MailCountry",
                    "value": country_map[mailing_address.country],
                    "required": True,
                    "length": 100,
                },
            ]
            if business_address != mailing_address
            else []
        )
        contact = Contacts(self.person.contact).preferredContact
        employer_contact = [
            {
                "action_type": Action.Input.value,
                "label": "Famil name",
                "id": "#syncA_App_Emp_ContactLname",
                "value": contact.last_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Given name",
                "id": "#syncA_App_Emp_ContactFname",
                "value": contact.first_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Job title",
                "id": "#syncA_App_Emp_ContactTitle",
                "value": contact.position,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Phone number",
                "id": "#syncA_App_Emp_ContactPhone",
                "value": contact.phone,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Email",
                "id": "#syncA_App_Emp_ContactEmail",
                "value": contact.email,
                "required": True,
                "length": 100,
            },
        ]
        with_joboffer = self.person.joboffer.noc != ""
        has_joboffer = [
            {
                "action_type": Action.Radio.value,
                "label": "Do you have an offer of full-time employment? ",
                "id": "#syncA_App_FullTimeEmpOffer-Yes"
                if with_joboffer
                else "#syncA_App_FullTimeEmpOffer-No",
            }
        ]
        indeterminate_joboffer = [
            {
                "action_type": Action.Radio.value,
                "label": "Do you have an indeterminate job offer? ",
                "id": "#syncA_App_FullTimeEmpOfferInd-Yes"
                if self.person.joboffer.permanent
                else "#syncA_App_FullTimeEmpOfferInd-No",
            }
        ]
        end_date = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "End date",
                    "id": "#syncA_App_Job_OfferEndDate",
                    "value": self.person.joboffer.work_end_date,
                    "required": True,
                    "length": 100,
                }
            ]
            if not self.person.joboffer.permanent
            else []
        )

        joboffer_details = [
            {
                "action_type": Action.Input.value,
                "label": "Job title",
                "id": "#syncA_App_Job_Title",
                "value": self.person.joboffer.job_title,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "NOC",
                "id": "#syncA_App_Job_NOC",
                "value": self.person.joboffer.noc,
                "required": False,
                "length": 100,
            },
            {
                "action_type": Action.Radio.value,
                "label": "Is the occupation regulated/licensed? ",
                "id": "#BCPNP_App_Job_IsRegulated-Yes"
                if self.person.joboffer.license_request
                else "#BCPNP_App_Job_IsRegulated-No",
            },
            {
                "action_type": Action.Input.value,
                "label": "Hours of work per week",
                "id": "#syncA_App_Job_HoursPerWeek\\ ",
                "value": self.person.joboffer.weekly_hours,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Hourly wage",
                "id": "#syncA_App_Job_HourlyWage",
                "value": self.person.joboffer.hourly_rate,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Annual Salary",
                "id": "#syncA_App_Job_AnnualWage",
                "value": self.person.joboffer.annual_rate,
                "required": True,
                "length": 100,
            },
        ]
        job_offer = has_joboffer + indeterminate_joboffer + end_date + joboffer_details
        work_addresses = Addresses(self.person.eraddress).workings
        locations = []
        for index, address in enumerate(work_addresses):
            values = [
                {
                    "action_type": Action.Input.value,
                    "label": "Unit number ",
                    "id": f"#syncA_App_Emp_WorkAddrUnit-{index}", # #BCPNP_App_Emp_WorkAddrUnit-{index}
                    "value": address.unit,
                    "required": False,
                    "length": 10,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Street address",
                    "id": f"#syncA_App_Job_WorkLocationAddr-{index}",
                    "value": address.street_address,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "City ",
                    "id": f"#syncA_App_Job_WorkLocationCity-{index}",
                    "value": address.city,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Phone number ",
                    "id": f"#syncA_App_Job_WorkLocationPhone-{index}",
                    "value": address.phone,
                },
            ]
            locations.append(values)
        work_locations = [
            {
                "action_type": Action.RepeatSection.value,
                "button_id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > div.tab-content > ng-transclude > uf-tab:nth-child(5) > div > div > uf-panel:nth-child(5) > div > uf-panel-body > div > div:nth-child(3) > uf-clone-repeatable > a > i",
                "value": locations,
            }
        ]

        # dashboard = DashboardApp()
        actions = (
            # dashboard.jump("Job Offer")
            company_details
            + business_location
            + is_same
            + mailing_location
            + employer_contact
            + job_offer
            + work_locations
            # + dashboard.save
        )
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Joboffer",
                "actions": actions,
                "id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > ul > li:nth-child(8) > a",
                "next_page_tag":"#BCPNP_App_HaspaidRep-Yes2"
            }
        ]



from datetime import date
from pydantic import BaseModel
from typing import Optional,List

class Status(BaseModel):
    current_country:str
    current_country_status:str
    current_workpermit_type:Optional[str]
    has_vr:Optional[bool]
    current_status_start_date:date
    current_status_end_date:date
    other_status_explaination:Optional[str]
    last_entry_date:Optional[date]
    last_entry_place:Optional[date]


from enum import Enum


class Action(Enum):
    Radio_Yes_No = "Radio_Yes_No"
    Radio_Yes_No_Null = "Radio_Yes_No_Null"
    Checkbox = "Checkbox"
    Input = "Input"
    Areatext = "Areatext"
    Select = "Select"
    Radio = "Radio"
    Button = "Button"
    Turnpage = "Turnpage"
    Wait4Element = "Wait4Elment"
    RepeatSection = "RepeatSection"
    DependantSelect = "DependantSelect"
    SelectPopup = "SelectPopup"
    SelectPopup2 = "SelectPopup2"
    GotoPage = "GotoPage"
    WebPage = "WebPage"
    Login = "Login"
    Security = "Security"
    Upload = "Upload"
    Pdf = "Pdf"
    PrPortalPick = "PrPortalPick"
    Wait = "Wait"
    Continue = "Continue"


class Role(Enum):
    PA = "Principal Applicant"
    SP = "Spouse"
    DP = "Dependant"



from .context import DATADIR
from typing import List

from pydantic import EmailStr
from model.common.commonmodel import CommonModel
from model.bcpnp.data import Bcpnp, General, JobOffer, ErAddress
from model.common.jobposition import Position
from model.common.rcic import Rcic
from model.common.advertisement import Advertisement, Advertisements, RecruitmentSummary
from model.common.person import Person, PersonalAssess
from model.common.address import Address, Addresses
from model.common.phone import Phone, Phones
from model.common.wordmaker import WordMaker
import os


class Personal(Person):
    email: EmailStr

    def __str__(self):
        return self.full_name


class EmployeeTrainingModel(CommonModel):
    eraddress: List[ErAddress]
    general: General
    position: Position
    personal: Personal
    phone: List[Phone]
    address: List[Address]
    joboffer: JobOffer
    personalassess: PersonalAssess
    bcpnp: Bcpnp
    rcic: Rcic
    advertisement: List[Advertisement]
    recruitmentsummary: RecruitmentSummary

    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self, excels=None, output_excel_file=None):
        if output_excel_file:
            excels = self.getExcels(
                [
                    "excel/er.xlsx",
                    "excel/pa.xlsx",
                    "excel/recruitment.xlsx",
                    "excel/bcpnp.xlsx",
                    "excel/rep.xlsx",
                ]
            )
        else:
            if excels is None and len(excels) == 0:
                raise ValueError(
                    "You must input excel file list as source data for validation"
                )
        # call parent class for validating
        super().__init__(excels, output_excel_file, globals())

    @property
    def residential_address(self):
        addresses = Addresses(self.address)
        return addresses.residential

    @property
    def work_location(self):
        addresses = Addresses(self.eraddress)
        return addresses.working

    @property
    def selected_contact(self):
        contacts = Contacts(self.contact)
        return contacts.preferredContact

    @property
    def advertisements(self):
        return Advertisements(self.advertisement)

    @property
    def phone_number(self):
        return Phones(self.phone).PreferredPhone

    @property
    def person(self):
        return {
            "full_name": self.personal.full_name,
            "attributive": self.personal.attributive,
            "object": self.personal.object,
            "subject": self.personal.subject,
            "short_name": self.personal.short_name,
            "email": self.personal.email,
            "phone": self.phone_number,
            "address": self.residential_address,
        }


class EmployeeTrainingDocxAdaptor:
    def __init__(self, employer_training_obj: EmployeeTrainingModel):
        self.employer_training_obj = employer_training_obj

    def re_generate_dict(self):
        summary_info = {
            "advertisement": self.employer_training_obj.advertisements,
            "personal": self.employer_training_obj.person,
            "date_of_offer": self.employer_training_obj.joboffer.date_of_offer,
            "work_start_date": self.employer_training_obj.joboffer.start_date_say,
            "joboffer_date": self.employer_training_obj.joboffer.date_of_offer,
            "work_location": self.employer_training_obj.work_location,
        }
        return {**self.employer_training_obj.dict(), **summary_info}

    def make(self, output_docx):
        template_path = os.path.abspath(
            os.path.join(DATADIR, "word/bcpnp_facts_brief_employee.docx")
        )
        wm = WordMaker(template_path, self.re_generate_dict(), output_docx)
        wm.make()



from pydantic import BaseModel
from typing import Optional,List,Union
from collections import namedtuple


class LanguageBase(BaseModel):
    reading:Optional[Union[int,float]]
    writting:Optional[Union[int,float]]
    listening:Optional[Union[int,float]]
    speaking:Optional[Union[int,float]]
    test_type:Optional[str]
    
    
    def __str__(self):
        return str(self.test_type)+f"(R:{self.reading} W:{self.writting} L: {self.listening} S: {self.speaking})"

class Languages(object):
    def __init__(self, language_list: List[LanguageBase]) -> None:
        self.languages=language_list
    
    def getSpecifiedLanguage(self,test_type):
        language=[language for language in self.languages if language.test_type==test_type]
        return language[0] if language else None
    
    @property
    def PreferredLanguage(self):
        for language_type in ['IELTS','CELPIP','TEF','TCF']:
            language=self.getSpecifiedLanguage(language_type)
            if language: return language



comment = {
    "info-general": {
        "pt_pt_employee_number": "每周工作时间少于30小时的为兼职",
    },
    "table-eraddress": {"mailing_address": "如果地址跟商业地址相同,请copy商业地址到这里"},
    "table-finance": {"year": "请填写上一年度,以及上上年度2年的财务信息"},
    "table-education": {"is_trade": "是否是技工类别？比如水电工"},
    "info-joboffer": {
        "ot_after_hours_unit": "加班工资开始计算的单位,比如,每天超过8小时,那么该单位就是day；如果每周超过40小时,那么单位就是week",
        "ot_ratio": "加班工资是平时工资的倍数,比如1.5",
        "vacation_pay_percentage": "一般是4%,就填写0.04即可",
        "duties": "详细职责描述,可将excel行拉长。每条职责一行,一般在别的编辑器里面写好copy过来即可。",
        "specific_edu_requirement": "一句话描述对学历、教育程度、相关专业的要求",
    },
    "table-family": {"relationship": "请填写你的: 配偶,子女,母亲,父亲,兄弟姐妹,包括同母异父/同母异父的兄弟姐妹,"},
    "table-marriage": {
        "sp_canada_status": "请选择:Citizen (公民),Visitor(访问者,包括旅游,探亲,商务等),Student(学生),Worker(工人),Other(其他)"
    },
    "info-personal": {
        "last_name": "保持跟护照一致都拼音姓",
        "first_name": "保持跟护照一致都拼音名",
        "uci": "加拿大签证上Unified Client Identity号码",
        "ita_assessed": "这个是加拿大行业培训机构(Industry Training Authority)",
        "english_french": "English(英语) French(法语)Both(两个都会) Neither(两个都不会)",
        "primary_school_years": "如果没有,请填写0",
        "secondary_school_years": "如果没有,请填写0",
        "post_secondary_school_years": "如果没有,请填写0",
        "other_school_years": "如果没有,请填写0",
    },
    "table-personid": {"expiry_date": "如果没有到期日期,请填写当前日期加上30年的日期"},
    "info-personalassess": {
        "work_experience_brief": "教育背景,工作经历,个人品质,语言能力 用于PNP的雇主信或者LMIA申请回答为什么申请人合格的问题。工作表现评价用于已经在该雇主那里工作的情况,在雇主推荐信中使用。"
    },
    "table-language": {
        "registration_number": "CELPIP叫registration number, IELTS叫 Test report form number, TEF叫Attestation n*, TCF叫N° de l'attestation",
        "pin": "仅仅当考试类型是CELPIP时需要",
        "date": "IELTS: administrator signature date,CELPIP: date of signing,TEF: date of session,TEF: date of session",
    },
    "info-marriage": {
        "married_date": "若是single则不需要填写",
        "pre_sp_last_name": "只填写最近一段婚姻",
    },
    "table-phone": {"cellular": "优选手机作为通讯选择。如果有手机,只填写手机即可,如果没有,依次可以选择家庭电话,工作电话"},
    "table-address": {"residential_address": "如果地址跟邮件地址相同,请copy邮件地址到这里"},
    "info-trcasein": {
        "doc_number": "学签/工签/VR的右上角黑色的,一般为1个字母+9位数字",
        "original_purpose": "Business-商业,Tourism-旅游,study-学习,work-工作,family visit-家庭访问,或者other-其它",
        "original_other_reason": "若上一行填other则将原因补充在这里",
        "submission_date": "可以不用填写",
    },
    "info-sp": {
        "dli": "中小学无需填写,大学参考学校给的offer",
        "tuition_cost": "填写一年的大致学费,加币,如49800",
        "room_cost": "填写一年的大致住宿费用,加币,如12000",
        "other_cost": "填写一年的大致生活等其它开支,加币,如12000",
        "other": "填写亲戚- relative,或奖学金-scholarship等",
    },
    "info-spincanada": {
        "dli": "中小学无需填写,大学参考学校给的offer",
        "tuition_cost": "填写一年的大致学费,加币,如49800",
        "room_cost": "填写一年的大致住宿费用,加币,如12000",
        "other_cost": "填写一年的大致生活等其它开支,加币,如12000",
        "other": "填写亲戚- relative,或奖学金-scholarship等",
    },
    "info-wpincanada": {"caq_number": "如果工作是在魁北克的,需要拿到CAQ号码,其他省份不需要"},
    "table-cor": {"start_date": " 第一行填写当前居住国"},
    "table-military": {
        "service_detail": "你的部队信息： 部门,番号,长官姓名",
        "combat_detail": "参加过的战斗的时间、地点",
        "reason_for_end": "说明结束这段服役经历的原因,比如退役...",
    },
    "table-employment": {
        "duties": "可以用鼠标把这个单元拉大,逐条列出你的所有工作职责,可以在其他编辑器中写好然后复制到这里",
        "noc_code": "这个客户不需要填写,由移民顾问填写",
    },
}



from webform.models.definition import Action
from webform.models.bcpnpmodel_reg import BcpnpModelReg, BcpnpEEModelReg
from webform.bcpnp.dashboard import DashboardReg
from model.common.educationbase import EducationHistory
from webform.bcpnp.data import getEducation, eca_map
from typing import Union


class Education:
    def __init__(self, person: Union[BcpnpModelReg, BcpnpEEModelReg]):
        self.person = person

    def fill(self):
        # get highest edu level
        highest_edu = EducationHistory(self.person.education).highestEducation
        is_trade = highest_edu.is_trade
        highest_section = [
            {
                "action_type": Action.Select.value,
                "lable": "Highest level of education",
                "id": "#BCPNP_App_Edu_HighestLevel",
                "value": getEducation(highest_edu.education_level, is_trade=True)
                if is_trade
                else getEducation(highest_edu.education_level),
            },
            {
                "action_type": Action.Input.value,
                "label": "Date highest level of education completed?",
                "value": highest_edu.end_date.strftime("%Y-%m-%d"),
                "id": "#BCPNP_App_Edu_HighestLevel_Loc",
            },
        ]
        edu_in_canada = [
            {
                "action_type": Action.Radio.value,
                "label": "Did you obtain this education in Canada? ",
                "id": "#BCPNP_App_Edu_HighestLevel_Can-Yes"
                if highest_edu.country == "Canada"
                else "#BCPNP_App_Edu_HighestLevel_Can-No",
            }
        ]
        edu_in_bc = (
            [
                {
                    "action_type": Action.Radio.value,
                    "label": "Did you obtain this education in British Columbia?",
                    "id": "#syncA_App_Edu_HighestLevel_BC-Yes"
                    if highest_edu.country == "Canada"
                    else "#syncA_App_Edu_HighestLevel_BC-No",
                }
            ]
            if highest_edu.country == "Canada"
            else []
        )

        has_eca = [
            {
                "action_type": Action.Radio.value,
                "label": "Do you have an Education Credential Assessment?  ",
                "id": "#BCPNP_App_Edu_CA-Yes"
                if self.person.personal.did_eca
                else "#BCPNP_App_Edu_CA-No",
            }
        ]
        eca = (
            [
                {
                    "action_type": Action.Select.value,
                    "lable": "Qualified suppliers",
                    "id": "#BCPNP_App_Edu_CA_Cert_Supplier",
                    "value": eca_map[self.person.personal.eca_supplier],
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Certificate number",
                    "value": highest_edu.end_date.strftime("%Y-%m-%d"),
                    "id": "#BCPNP_App_Edu_CA_CertNo",
                },
            ]
            if self.person.personal.did_eca
            else [
                {
                    "action_type": Action.Radio.value,
                    "lable": "Has the Industry Training Authority (ITA) assessed your training and experience?",
                    "id": "#BCPNP_App_Edu_ITA-Yes"
                    if self.person.personal.ita_assessed
                    else "#BCPNP_App_Edu_ITA-No",
                }
            ]
        )

        ita_assessed = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Certificate number",
                    "value": self.person.personal.ita_assess_number,
                    "id": "#BCPNP_App_Edu_ITA_Cert_No",
                }
            ]
            if self.person.personal.ita_assessed
            else []
        )
        if self.person.personal.did_eca:
            ita_assessed = []

        # dashboard = DashboardReg()
        actions = (
            # dashboard.jump("Education")
            highest_section
            + edu_in_canada
            + edu_in_bc
            + has_eca
            + eca
            + ita_assessed
            # + dashboard.save
        )
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Education",
                "actions": actions,
                "id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > ul > li:nth-child(3) > a",
                "next_page_tag":"#BCPNP_App_Job_Work_ExperienceRelated-No"
            }
        ]



from webform.models.definition import Action, Role
from webform.models.prmodel import PrModel
from model.common.address import Addresses
from model.common.phone import Phones
from model.common.id import IDs
from model.common.educationbase import EducationHistory
from typing import List
from webform.prportal.data.d0008 import *
from webform.prportal.data.citizen import citizen
from webform.prportal.data.country_birth import country_birth
from webform.prportal.data.country_residence import country_residence
from webform.models.prmodel import CORs


class F0008:
    def __init__(self, pa: PrModel, sp: PrModel | None, dps: List[PrModel]):
        self.pa = pa
        self.sp = sp
        self.dps = dps

    @property
    def pdf(self):
        return [{"action_type": Action.Pdf.value}]

    def goTo0008(self):
        return [
            {
                "action_type": "WebPage",
                "page_name": "Start/Edit for 0008 form",
                "actions": [],
                "next_page_tag":"body > pra-root > pra-localized-app > main > div > pra-imm0008-page1 > div > a.btn.btn-primary",
                "id": "body > pra-root > pra-localized-app > main > div > pra-intake-landing-page > pra-web-form-table > div > table > tbody > tr:nth-child(1) > td:nth-child(5) > button",
            },
            {
                "action_type": "WebPage",
                "page_name": "Start/Edit for 0008 form, Continue",
                "actions": [],
                "next_page_tag":"#city",
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm0008-page1 > div > a.btn.btn-primary",
            },
        ]

    def saveContinue(self, page):
        return f"body > pra-root > pra-localized-app > main > div > pra-imm0008-page{page} > div > button"

    def saveContinueDep(self):
        return "body > pra-root > pra-localized-app > main > div > pra-imm0008-dependant > div.buttons-container > button"

    def saveContinueDep1(self):
        return "body > pra-root > pra-localized-app > main > div > pra-imm0008-dependant > div.buttons-container > button.btn.btn-primary.ng-star-inserted"

    def completeBack(self):
        return [
            {
                "action_type": "WebPage",
                "page_name": "Complete and return to application",
                "actions": [],
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm0008 > lib-navigation-buttons > div > button.btn.btn-secondary",
            }
        ]

    # page 2 application details
    def applicationDetails(self):
        actions = [
            {
                "action_type": "Select",
                "label": "Correspondence",
                "value": english_french_map[self.pa.prcase.communication_language],
                "id": "#correspondence",
            },
            {
                "action_type": "Select",
                "label": "Interview",
                "value": all_languages[self.pa.prcase.interview_language],
                "id": "#interview",
            },
            {
                "action_type": "Select",
                "label": "Interpreter requested",
                "value": interpreter_map["Yes"]
                if self.pa.prcase.need_translator
                else interpreter_map["No"],
                "id": "#interpreterRequested",
            },
            {
                "action_type": "Select",
                "label": "Province",
                "value": province_map[self.pa.prcase.intended_province],
                "id": "#province",
            },
            {
                "action_type": "Select",
                "label": "City",
                "value": city_map[self.pa.prcase.intended_province][
                    self.pa.prcase.intended_city
                ],
                "id": "#city",
            },
        ] + self.pdf
        return [
            {
                "action_type": "WebPage",
                "page_name": "Application details",
                "actions": actions,
                "next_page_tag":"#personalDetailsForm-previouslyMarriedOrCommonLaw-no",
                "id": self.saveContinue(2),
            }
        ]

    # page 3 personal details
    def makePreviousCOR(self,person):
        value = []
        cor = person.cor
        cor.pop(0)  # the first line is current cor
        for index, c in enumerate(cor):
            block = [
                {
                    "action_type": "Select",
                    "label": "Country",
                    "value": country_residence[c.country],
                    "id": "#personalDetailsForm-prevCountry" + str(index),
                },
                {
                    "action_type": "Select",
                    "label": "Immigration status",
                    "value": immigration_status[c.status],
                    "id": "#personalDetailsForm-prevImmigrationStatus" + str(index),
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Start date",
                    "id": "#personalDetailsForm-prevStartDateOfImmigrationStatus"
                    + str(index),
                    "value": c.start_date.strftime("%Y/%m/%d"),
                    "length": 10,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "End date",
                    "id": "#personalDetailsForm-prevEndDateOfImmigrationStatus"
                    + str(index),
                    "value": c.end_date.strftime("%Y/%m/%d"),
                    "length": 10,
                    "required": True,
                },
            ]
            other = [
                {
                    "action_type": Action.Input.value,
                    "label": "Other explanation",
                    "id": "#personalDetailsForm-prevOtherImmigrationStatus"
                    + str(index),
                    "value": c.explanation,
                    "length": 30,
                    "required": True,
                }
            ]
            block = block + other if c.status == "Other" else block
            value.append(block)
        return [
            {
                "action_type": Action.RepeatSection.value,
                "button_text": "Add another",
                "value": value[0:2],  # max is 2
            }
        ]

    # page 3 personal details
    def personalDetails(self, person: PrModel, role: Role):
        relationship2pa = {
            "action_type": "Select",
            "label": "Relationship",
            "value": relationship_to_pa[person.personal.relationship_to_pa]
            if person.personal.relationship_to_pa
            else None,
            "id": "#dependantDetailsForm-relationshipToPA",
            "delay":1500
        }
        sp_dp = (
            [
                {
                    "action_type": "Radio",
                    "label": "Accompany to Canada",
                    "id": "#dependantDetailsForm-accompanyingPA-yes"
                    if person.personal.accompany_to_canada
                    else "#dependantDetailsForm-accompanyingPA-no"
                    
                },
                relationship2pa,
            ]
            if role != Role.PA
            else []
        )

        dependant_type_set = (
            [
                {
                    "action_type": "Select",
                    "label": "Dependant type",
                    "value": dependant_type[person.personal.dependant_type]
                    if person.personal.dependant_type
                    else None,
                    "id": "#dependantDetailsForm-dependantType",
                }
            ]
            if role != Role.PA
            and person.personal.relationship_to_pa
            in [
                "Adopted Child",
                "Child",
                "Grandchild",
                "Step-Child",
                "Step-Grandchild",
                "Parent",
                "Adoptive Parent",
            ]
            else []
        )

        dependant_type_set = (
            [
                dependant_type_set[0]
            ]
            if dependant_type_set
            else []
        )  # 如果是spouse，common-law则没有后续选择
        dependant_type_set = (
            dependant_type_set if role != Role.PA else []
        )  # 如果是PA， 则没有dependant_type_set

        name = [
            {
                "action_type": "Input",
                "label": "Family name",
                "value": person.personal.last_name,
                "id": "#personalDetailsForm-familyName",
                "length": 100,
                "required": True,
            },
            {
                "action_type": "Input",
                "label": "Given name",
                "value": person.personal.first_name,
                "id": "#personalDetailsForm-givenName",
                "length": 100,
                "required": True,
            },
        ]
        has_used_name = True if person.personal.used_first_name else False
        used_name = [
            {
                "action_type": "Radio",
                "label": "Have used name?",
                "value": has_used_name,
                "id": "#personalDetailsForm-usedOtherName-yes"
                if has_used_name
                else "#personalDetailsForm-usedOtherName-no",
            }
        ]
        used_name += (
            [
                {
                    "action_type": "Input",
                    "label": "Used family name",
                    "value": person.personal.used_last_name,
                    "id": "#personalDetailsForm-otherFamilyName",
                    "length": 100,
                    "required": True,
                },
                {
                    "action_type": "Input",
                    "label": "Used given name",
                    "value": person.personal.used_first_name,
                    "id": "#personalDetailsForm-otherGivenName",
                    "length": 100,
                    "required": True,
                },
            ]
            if has_used_name
            else []
        )
        uci = [
            {
                "action_type": "Input",
                "label": "UCI",
                "value": person.personal.uci,
                "id": "#personalDetailsForm-uci",
                "length": 100,
                "required": False,
            }
        ]
        physical_characteristics = [
            {
                "action_type": "Select",
                "label": "Sex",
                "value": sex_map[person.personal.sex],
                "id": "#personalDetailsForm-sex",
            },
            {
                "action_type": "Select",
                "label": "Eye color",
                "value": eye_color[person.personal.eye_color],
                "id": "#personalDetailsForm-eyeColour",
            },
            {
                "action_type": "Input",
                "label": "Height",
                "value": str(person.personal.height),
                "id": "#personalDetailsForm-heightInCM",
                "length": 3,
                "required": True,
            },
        ]
        birth_info = [
            {
                "action_type": "Input",
                "label": "DOB",
                "value": person.personal.dob.strftime("%Y/%m/%d"),
                "id": "#personalDetailsForm-dob",
                "length": 10,
                "required": True,
            },
            {
                "action_type": "Input",
                "label": "Place of birth",
                "value": person.personal.place_of_birth,
                "id": "#personalDetailsForm-cityOfBirth",
                "length": 30,
                "required": True,
            },
            {
                "action_type": "Select",
                "label": "Country of birth",
                "value": country_birth[person.personal.country_of_birth],
                "id": "#personalDetailsForm-countryOfBirth",
            },
        ]
        citizens = [
            {
                "action_type": "Select",
                "label": "Citizen 1",
                "value": citizen[person.personal.citizen],
                "id": "#personalDetailsForm-citizenship1",
            },
            {
                "action_type": "Select",
                "label": "Citizen 2",
                "value": citizen[person.personal.citizen2]
                if person.personal.citizen2
                else None,
                "id": "#personalDetailsForm-citizenship2",
            },
        ]
        ccor = CORs(person.cor).current
        current_cor = [
            {
                "action_type": "Select",
                "label": "Current country of residence",
                "value": country_residence[ccor.country],
                "id": "#personalDetailsForm-currentCountry",
            },
            {
                "action_type": "Select",
                "label": "Status",
                "value": immigration_status[ccor.status],
                "id": "#personalDetailsForm-immigrationStatus",
            },
        ]
        ccor_period = [
            {
                "action_type": "Input",
                "label": "From",
                "value": ccor.start_date.strftime("%Y/%m/%d"),
                "id": "#personalDetailsForm-startDateofImmigrationStatus",
                "length": 10,
                "required": True,
            },
            {
                "action_type": "Input",
                "label": "To",
                "value": ccor.end_date.strftime("%Y/%m/%d"),
                "id": "#personalDetailsForm-endDateOfImmigrationStatus",
                "length": 10,
                "required": True,
            },
        ]
        last_entry = [
            {
                "action_type": "Input",
                "label": "Date of last entry",
                "value": person.personal.last_entry_date.strftime("%Y/%m/%d")
                if person.personal.last_entry_date
                else None,
                "id": "#personalDetailsForm-dateOfLastEntry",
                "length": 10,
                "required": True,
            },
            {
                "action_type": "Input",
                "label": "Place of last entry",
                "value": person.personal.last_entry_place,
                "id": "#personalDetailsForm-placeOfLastEntry",
                "length": 30,
                "required": True,
            },
        ]
        other = [
            {
                "action_type": "Input",
                "label": "details",
                "value": person.personal.other_explanation,
                "id": "#personalDetailsForm-otherImmigrationStatus",
                "length": 30,
                "required": True,
            }
        ]

        match ccor.status:
            case "Citizen" | "Permanent Resident":
                current_cor += last_entry if ccor.country == "Canada" else current_cor
            case "Worker" | "Student" | "Visitor":
                current_cor += (
                    ccor_period + last_entry
                    if ccor.country == "Canada"
                    else ccor_period
                )
            case "Refugess":
                current_cor += last_entry if ccor.country == "Canada" else current_cor
            case "Other":
                current_cor += (
                    ccor_period + other + last_entry
                    if ccor.country == "Canada"
                    else ccor_period + other
                )

        has_previous_cor = True if len(person.cor) > 1 else False
        previous_cor = [
            {
                "action_type": "Radio",
                "label": "Previous countries of residence",
                "id": "#personalDetailsForm-hasPreviousCountries-yes"
                if has_previous_cor
                else "#personalDetailsForm-hasPreviousCountries-no",
            }
        ]
        previous_cor = (
            previous_cor + self.makePreviousCOR(person) if has_previous_cor else previous_cor
        )
        # marriage
        marital_status = [
            {
                "action_type": "Select",
                "label": "Current marital status",
                "value": current_marital_status[person.marriage.marital_status],
                "id": "#personalDetailsForm-maritalStatus",
            }
        ]
        married_date = [
            {
                "action_type": Action.Input.value,
                "label": "Married date",
                "id": "#personalDetailsForm-dateOfMarriageOrCommonLaw",
                "value": person.marriage.married_date.strftime("%Y/%m/%d")
                if person.marriage.married_date
                else None,
                "length": 10,
                "required": True,
            }
        ]
        spouse_name = [
            {
                "action_type": Action.Input.value,
                "label": "Family name",
                "id": "#personalDetailsForm-familyNameOfSpouse",
                "value": person.marriage.sp_last_name,
                "length": 100,
                "required": True,
            },
            {
                "action_type": Action.Input.value,
                "label": "Given name",
                "id": "#personalDetailsForm-givenNameOfSpouse",
                "value": person.marriage.sp_first_name,
                "length": 100,
                "required": True,
            },
        ]
        match person.marriage.marital_status:
            case "Annulled Marriage" | "Divorced" | "Separated" | "Single" | "Unknown" | "Widowed":
                pass
            case "Common-Law" | "Married":
                marital_status += married_date + spouse_name
        # previous marriage
        has_pre_marriage = [
            {
                "action_type": "Radio",
                "label": "Has previous marriage",
                "id": "#personalDetailsForm-previouslyMarriedOrCommonLaw-yes"
                if person.marriage.previous_married
                else "#personalDetailsForm-previouslyMarriedOrCommonLaw-no",
            }
        ]
        
        pre_spouse_dob=[{
                "action_type": Action.Input.value,
                "label": "Date of birth",
                "id": "#previousRelationshipForm-previousSpouseDob",
                "value": person.marriage.pre_sp_dob.strftime("%Y/%m/%d")
                if person.marriage.pre_sp_dob
                else None,
                "length": 10,
                "required": True,
            }] if role==Role.PA else []
        
        pre_marriage = [
            {
                "action_type": Action.Input.value,
                "label": "Family name",
                "id": "#previousRelationshipForm-previousSpouseFamilyName",
                "value": person.marriage.pre_sp_last_name,
                "length": 100,
                "required": True,
            },
            {
                "action_type": Action.Input.value,
                "label": "Given name",
                "id": "#previousRelationshipForm-previousSpouseGivenName",
                "value": person.marriage.pre_sp_first_name,
                "length": 100,
                "required": True,
            },
            *pre_spouse_dob,
            {
                "action_type": "Select",
                "label": "Type of relationship",
                "value": previous_marital_status[person.marriage.pre_relationship_type]
                if person.marriage.pre_relationship_type
                else None,
                "id": "#previousRelationshipForm-typeOfRelationship",
            },
            {
                "action_type": Action.Input.value,
                "label": "Start date of relationship",
                "id": "#previousRelationshipForm-startDateofRelationship",
                "value": person.marriage.pre_start_date.strftime("%Y/%m/%d")
                if person.marriage.pre_start_date
                else None,
                "length": 10,
                "required": True,
            },
            {
                "action_type": Action.Input.value,
                "label": "End date of relationship",
                "id": "#previousRelationshipForm-endDateOfRelationship",
                "value": person.marriage.pre_end_date.strftime("%Y/%m/%d")
                if person.marriage.pre_end_date
                else None,
                "length": 10,
                "required": True,
            },
        ]
        has_pre_marriage += pre_marriage if person.marriage.previous_married else []
        # summary
        save_continue_id = (
            self.saveContinue(3) if role == Role.PA else self.saveContinueDep()
        )

        actions = (
            sp_dp
            + dependant_type_set
            + name
            + used_name
            + uci
            + physical_characteristics
            + birth_info
            + citizens
            + current_cor
            + previous_cor
            + marital_status
            + has_pre_marriage
            + self.pdf
        )
        return [
            {
                "action_type": "WebPage",
                "page_name": "Page 3 personal details"
                if role == Role.PA
                else "Dependant personal details",
                "actions": actions,
                "next_page_tag":"#contactNo" if role==Role.PA else "#intendedOccupation",
                "id": save_continue_id,
            }
        ]

    def makeAddress(self, address, type):
        types = {
            "mailing": {
                "unit": "#MailingAptUnit",
                "street_number": "#MailingStreetNum",
                "street_name": "#MailingStreetName",
                "city": "#MailingCityTown",
                "country": "#MailingCountry",
                "province": "#MailingProvinceState",
                "country": "#MailingCountry",
                "post_code": "#MailingPostalCode",
                "district": "#MailingDistrict",
            },
            "residential": {
                "unit": "#ResidentialAptUnit",
                "street_number": "#ResidentialStreetNum",
                "street_name": "#ResidentialStreetName",
                "city": "#ResidentialCityTown",
                "country": "#ResidentialCountry",
                "province": "#ResidentialProvinceState",
                "post_code": "#ResidentialPostalCode",
                "district": "#ResidentialDistrict",
            },
        }
        the_address = [
            {
                "action_type": Action.Input.value,
                "label": "Apt/Unit",
                "id": types[type]["unit"],
                "value": address.unit if address.unit else None,
                "length": 10,
                "required": True,
            },
            {
                "action_type": Action.Input.value,
                "label": "Street number",
                "id": types[type]["street_number"],
                "value": address.street_number if address.street_number else None,
                "length": 10,
                "required": True,
            },
            {
                "action_type": Action.Input.value,
                "label": "Street name",
                "id": types[type]["street_name"],
                "value": address.street_name if address.street_name else None,
                "length": 100,
                "required": True,
            },
            {
                "action_type": Action.Input.value,
                "label": "City",
                "id": types[type]["city"],
                "value": address.city if address.city else None,
                "length": 30,
                "required": True,
            },
            {
                "action_type": "Select",
                "label": "Country",
                "value": country_residence[address.country]
                if address.country
                else None,
                "id": types[type]["country"],
            },
        ]
        province = [
            {
                "action_type": "Select",
                "label": "Province",
                "value": province_map[address.province] if address.province else None,
                "id": types[type]["province"],
            }
        ]
        post_code = [
            {
                "action_type": Action.Input.value,
                "label": "Post code",
                "id": types[type]["post_code"],
                "value": address.post_code if address.post_code else None,
                "length": 7,
                "required": True,
            }
        ]
        district = [
            {
                "action_type": Action.Input.value,
                "label": "District",
                "id": types[type]["district"],
                "value": address.district if address.district else None,
                "length": 100,
                "required": True,
            }
        ]
        the_address += (
            province + post_code
            if address.country == "Canada"
            else post_code + district
        )
        return the_address

    # page 4 contact information
    def contactInfo(self):
        addresses = Addresses(self.pa.address)
        mailing = addresses.mailing
        residential = addresses.residential
        po = [
            {
                "action_type": Action.Input.value,
                "label": "PO box",
                "id": "#mailingPOBox",
                "value": mailing.po_box if mailing.po_box else None,
                "length": 15,
                "required": False,
            }
        ]
        is_same_address = [
            {
                "action_type": "Radio",
                "label": "Same address?",
                "id": "#yes" if mailing == residential else "#no",
            }
        ]
        mailing_address = po + self.makeAddress(mailing, "mailing")
        residential_address = (
            self.makeAddress(residential, "residential")
            if mailing != residential
            else []
        )

        phone = Phones(self.pa.phone).PreferredPhone
        primary_phone = [
            {
                "action_type": "Radio",
                "label": "Canada/US or other",
                "id": "#primaryNA" if phone.isCanadaUs else "#primaryOther",
            },
            {
                "action_type": "Select",
                "label": "Type",
                "value": phone_type[phone.variable_type],
                "id": "#PrimaryType",
            },
        ]
        country_code = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Country code",
                    "id": "#PrimaryCountryCode",
                    "value": phone.country_code,
                    "length": 4,
                    "required": False,
                }
            ]
            if not phone.isCanadaUs
            else []
        )
        number_ext = [
            {
                "action_type": Action.Input.value,
                "label": "Country code",
                "id": "#PrimaryNumber",
                "value": "-".join(phone.NA_format_list)
                if phone.isCanadaUs
                else phone.number,
                "length": 20,
                "required": False,
            },
            {
                "action_type": Action.Input.value,
                "label": "Extension",
                "id": "#PrimaryExtension",
                "value": phone.ext,
                "length": 5,
                "required": False,
            },
        ]
        contact_using_the_email = [
            {
                "action_type": "Radio",
                "label": "Contact using the email in account",
                "id": "#contactYes",
            }
        ]
        primary_phone += country_code + number_ext
        actions = (
            mailing_address
            + is_same_address
            + residential_address
            + primary_phone
            + contact_using_the_email
            + self.pdf
        )
        return [
            {
                "action_type": "WebPage",
                "page_name": "Page 4: Contact information ",
                "actions": actions,
                "next_page_tag":'#validPassportNo',
                "id": self.saveContinue(4),
            }
        ]

    def passport_id(self, person: PrModel, role: Role):
        ids = IDs(person.personid)
        has_passport = [
            {
                "action_type": Action.Radio.value,
                "label": "Passport / travel document",
                "id": "#validPassportYes"
                if ids.passport.number
                else "#validPassportNo",
            }
        ]
        has_id = [
            {
                "action_type": Action.Radio.value,
                "label": "Has national id",
                "id": "#NICYes" if ids.national_id.number else "#NICNo",
            }
        ]
        passport_content = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Passport number",
                    "id": "#passportNumber",
                    "value": ids.passport.number,
                    "length": 20,
                    "required": True,
                },
                {
                    "action_type": "Select",
                    "label": "Country of issue",
                    "value": country_residence[ids.passport.country]
                    if ids.passport.country
                    else None,
                    "id": "#countryOfIssue",
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Issue date",
                    "id": "#issueDate",
                    "value": ids.passport.issue_date.strftime("%Y/%m/%d"),
                    "length": 10,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Expiry date",
                    "id": "#expiryDate",
                    "value": ids.passport.expiry_date.strftime("%Y/%m/%d"),
                    "length": 10,
                    "required": True,
                },
            ]
            if ids.passport.number
            else []
        )
        id_content = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "ID number",
                    "id": "#nationalIdentityNumber",
                    "value": ids.national_id.number,
                    "length": 20,
                    "required": True,
                },
                {
                    "action_type": "Select",
                    "label": "Country of issue",
                    "value": country_residence[ids.national_id.country]
                    if ids.national_id.country
                    else None,
                    "id": "#countryOfIssue",
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Issue date",
                    "id": "#issueDate",
                    "value": ids.national_id.issue_date.strftime("%Y/%m/%d")
                    if ids.national_id.issue_date
                    else None,
                    "length": 10,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Expiry date",
                    "id": "#expiryDate",
                    "value": ids.national_id.expiry_date.strftime("%Y/%m/%d")
                    if ids.national_id.expiry_date
                    else None,
                    "length": 10,
                    "required": True,
                },
            ]
            if ids.national_id.number
            else []
        )

        passport = has_passport + passport_content if ids.passport else has_passport
        national_id = has_id + id_content if ids.national_id else has_id
        passport_continue_id = (
            self.saveContinue(5) if role == Role.PA else self.saveContinueDep1()
        )
        national_id_continue_id = (
            self.saveContinue(6) if role == Role.PA else self.saveContinueDep1()
        )
        return [
            {
                "action_type": "WebPage",
                "page_name": "Page 5: Passport"
                if role == Role.PA
                else "Dependant passport",
                "actions": passport + self.pdf,
                "next_page_tag":"#NICNo",
                "id": passport_continue_id,
            },
            {
                "action_type": "WebPage",
                "page_name": "Page 6: National ID"
                if role == Role.PA
                else "Dependant national id",
                "actions": national_id + self.pdf,
                "next_page_tag":"#intendedOccupation" if role==Role.PA else "body > pra-root > pra-localized-app > main > div > pra-imm0008-page9 > pra-imm0008-view-dependant-form > button",
                "id": national_id_continue_id,
            },
        ]

    def education(self, person: PrModel, role: Role):
        edu = EducationHistory(person.education)
        highest_eud = edu.highest
        years = (
            person.personal.primary_school_years
            + person.personal.secondary_school_years
            + person.personal.post_secondary_school_years
            + person.personal.other_school_years
        )
        education_set = [
            {
                "action_type": "Select",
                "label": "Education level",
                "value": education_level[highest_eud] if highest_eud else None,
                "id": "#educationLevel",
            },
            {
                "action_type": Action.Input.value,
                "label": "Number of education years",
                "id": "#numberOfYear",
                "value": str(years),
                "length": 2,
                "required": True,
            },
            {
                "action_type": Action.Input.value,
                "label": "Current occupation",
                "id": "#currentOccupation",
                "value": self.pa.personal.current_occupation,
                "length": 50,
                "required": True,
            },
            {
                "action_type": Action.Input.value,
                "label": "Intended occupation",
                "id": "#intendedOccupation",
                "value": self.pa.personal.intended_occupation,
                "length": 50,
                "required": True,
            },
        ]
        save_continue_id = (
            self.saveContinue(7) if role == Role.PA else self.saveContinueDep1()
        )
        return [
            {
                "action_type": "WebPage",
                "page_name": "Page 7 Education"
                if role == Role.PA
                else "Dependant education",
                "actions": education_set + self.pdf,
                "next_page_tag":"#testing-no",
                "id": save_continue_id,
            }
        ]

    def language(self, person: PrModel, role: Role):
        language = [
            {
                "action_type": "Select",
                "label": "Native language",
                "value": all_languages[person.personal.native_language],
                "id": "#nativeLanguage",
            },
            {
                "action_type": "Select",
                "label": "English or French ",
                "value": english_french[person.personal.english_french],
                "id": "#language",
            },
        ]
        which_one_better = (
            [
                {
                    "action_type": "Select",
                    "label": "Which one is better ",
                    "value": english_french_map[person.personal.which_one_better]
                    if person.personal.which_one_better
                    else None,
                    "id": "#preferredLanguage",
                }
            ]
            if person.personal.english_french == "Both"
            else []
        )

        taken_test = [
            {
                "action_type": Action.Radio.value,
                "label": "Taken language test",
                "id": "#testing-yes"
                if person.personal.language_test
                else "#testing-no",
            }
        ]
        save_continue_id = (
            self.saveContinue(8) if role == Role.PA else self.saveContinueDep1()
        )
        return [
            {
                "action_type": "WebPage",
                "page_name": "Page 8 language detail"
                if role == Role.PA
                else "Dependant language detail",
                "actions": language + which_one_better + taken_test + self.pdf,
                "next_page_tag":"body > pra-root > pra-localized-app > main > div > pra-imm0008-page9 > div.buttons-container > button" if role==Role.PA else "#validPassportNo",
                "id": save_continue_id,
            }
        ]

    def gotoPage(self, page_num: int):
        pages = []
        for page in range(2, page_num):
            pages += [
                {
                    "action_type": "WebPage",
                    "page_name": f"Page {page}",
                    "actions": [],
                    "id": self.saveContinue(page),
                }
            ]
        return pages

    def gotoPageDependants(self):
        return self.gotoPage(9)

    def addDependants(self):
        actions = [
            {
                "action_type": Action.Radio.value,
                "label": "Do you have dependants to add",
                "id": "#dependantNo"
                if not self.sp and len(self.dps) == 0
                else "#dependantYes",
            }
        ]
        return [
            {
                "action_type": "WebPage",
                "page_name": "Add dependants or return to applicaton",
                "actions": actions,
                "next_page_tag":"#dependantDetailsForm-accompanyingPA-yes",
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm0008-page9 > pra-imm0008-view-dependant-form > button"
                if self.sp or len(self.dps) > 0
                else "body > pra-root > pra-localized-app > main > div > pra-imm0008-page9 > div.buttons-container > button",
            }
        ]

    def fill(self):
        pa_form = (
            self.applicationDetails()
            + self.personalDetails(self.pa, Role.PA)
            + self.contactInfo()
            + self.passport_id(self.pa, Role.PA)
            + self.education(self.pa, Role.PA)
            + self.language(self.pa, Role.PA)
        )

        sp_form = (
            self.addDependants()
            + self.personalDetails(self.sp, Role.SP)
            + self.education(self.sp, Role.SP)
            + self.language(self.sp, Role.SP)
            + self.passport_id(self.sp, Role.SP)
            if self.sp
            else []
        )
        dp_forms = []
        for dp in self.dps:
            if dp.personal.age >= 18:
                dp_forms += (
                    self.addDependants()
                    + self.personalDetails(dp, Role.DP)
                    + self.education(dp, Role.DP)
                    + self.language(dp, Role.DP)
                    + self.passport_id(dp, Role.DP)
                )

        return self.goTo0008() + pa_form + sp_form + dp_forms

    def fill_pa(self):
        return (
            self.goTo0008()
            + self.applicationDetails()
            + self.personalDetails(self.pa, Role.PA)
            + self.contactInfo()
            + self.passport_id(self.pa, Role.PA)
            + self.education(self.pa, Role.PA)
            + self.language(self.pa, Role.PA)
        )

    def fill_dp(self):
        sp_form = (
            self.addDependants()
            + self.personalDetails(self.sp, Role.SP)
            + self.education(self.sp, Role.SP)
            + self.language(self.sp, Role.SP)
            + self.passport_id(self.sp, Role.SP)
            if self.sp
            else []
        )
        dp_forms = []
        for dp in self.dps:
            # if dp.personal.age >= 18:
            dp_forms += (
                self.addDependants()
                + self.personalDetails(dp, Role.DP)
                + self.education(dp, Role.DP)
                + self.language(dp, Role.DP)
                + self.passport_id(dp, Role.DP)
            )
        return self.goTo0008() + self.gotoPageDependants() + sp_form + dp_forms
